function [xk, fk, Hfk, stop, trace] = direct(H, func, x0, A, b, eq, opts, varargin)
% Direct 1. order algorithm, with inexact gradient information
%
% Call
%    [xk, fk, Hfk, trace] = direct(H, func, x0, A, b, eq, opts, P1, P2, ...)
%
% Input
%
% H       : merit function for vector functions
% func    : function handle to the model
% x0      : starting point
% A, b    : linear constraints, A * x <= b
% eq      : number of (the first) rows of the linear constraints
%           which are equality constraints
% opts    : options from smopts, if left empty the default values are used
% P1, P2, ... : optional parameters passed to the model
%
%
% Output
%
% xk      : solution reached by the algorithm
% fk      : the model response at the solution
% Hfk     : H(fk)
% stop    : reason for stopping the algorithm
%              1  step length too small
%              2  reached the maximum allowed number of function evaluations
%              3  trust region too small
%              4  no decrease in local model
% trace   : trace of the iteration process

% js@imm.dtu.dk, January 27, 2003

% check if options are provided
if nargin < 7, opts = []; end

% otherwise get them from the default values
if isempty(opts), opts = smopts; end

if nargin < 6, eq = 0; end
if nargin < 5, A = []; b = []; end
if nargin < 3, error('No enough input arguments. At least four input arguments should be supplied.'); end

% determine the number of variables
n = size(x0, 1);

% set the initial size of the trust region
dx = opts.dx;
dxmax = 1e3 * (1 + max(dx, norm(x0)));  % max trust region size

% success indicator (the algorithm continues while stop is false (zero))
stop = 0;         

% model fct. evaluation counter
k = 0;

% starting point for the algorithm
x = x0;

% finite difference approximation of the model at x
if opts.display > 1
  disp('evaluating model')
end
[f, D] = fdgrad(func, x, [], 0, varargin{:});
Hf = smmerit(H, f);
k = k + 1 + n;

% define current iterate before entering the main loop
xk = x;
xn = 1 + norm(x);
fk = f;
Hfk = Hf;

% display some progress information
if opts.display, disp(sprintf('k: %d  Hf: %g', k, Hf)); end

if nargout > 3
  trace = struct('k', k, 'x', x, 'f', f, 'h', norm(x-x0, inf), 'dx', dx, 'rho', [], 'obj', Hf); % trace variable
end

if k >= opts.kmax
  stop = 2;
end

%%% main loop %%%

uphill = 0;

while ~stop
  
  % optimize linear model subject to linear constraints
  % (user and trust region constraints)
    
  if opts.display >= 2
    disp('Solving min_x H( D * (x - x_k) + f(x_k) )')
  end
    
  [x, s, Hs] = smtrlinear(H, D, xk, fk, dx, A, b, eq, opts);
    
  h = x - xk;
  
  % check if step length is too small
  if norm(h) <= opts.epsilon * xn
    stop = 1;
    break  % break the loop now
  end
  
  % DHs = -Delta S_k(x) (= H(f(x_k)) - H(s_k(x_k + h)))
  DHs = Hfk - Hs;
  
  if DHs <= 0
    % step is not accepted due to insufficient decrease in combined model
    
    stop = 4;  % no decrease in local model
    break
  end     
  
  % evaluate model at x
  if opts.display > 1
    disp('evaluating model')
  end
  f = feval(func, x, varargin{:});
  Hf = smmerit(H, f);
  k = k + 1;
  
  % DHf = H(f(x_k)) - H(f(x_k + h))
  DHf = Hfk - Hf;
  
  % boolean determining if the tentative step is accepted
  accept = (DHf >= opts.delta1 * DHs);
  
  % Update trust region
  
  % calculate gain
  rho = DHf / DHs;
  
  % set new trust region size
  if rho <= opts.delta2
    dx = opts.k1 * dx;
    % check if trust region is smaller than minimum allowed step length
    if dx <= opts.epsilon * xn
      stop = 3;  % stop before next iteration
    end
  end
  if rho >= opts.delta3, dx = min(dxmax, opts.k2 * dx); end
  
  % Broyden update of the linear approximation of the fine model
  Dnew = broyunfac(D, xk, x, fk, f);
  if opts.display > 2, disp(sprintf('||D-Dnew|| = %g', norm(D-Dnew, 'fro'))); end
  D = Dnew;
  
  if accept
    % accept the tentative iterate
    xk = x;
    xn = 1 + norm(x);
    fk = f;
    Hfk = Hf;

    if opts.display > 1, disp('step accepted'); end
    
  else
    
    if opts.display > 1, disp('step not accepted'); end
    
    if opts.dofinitediff
      uphill = uphill + 1;
      if uphill >= opts.maxuphill & opts.maxuphill > 0
        if opts.display > 2, disp('finite difference'); end
        [fk D] = fdgrad(fine, xk, fk, 0, varargin{:});
        k = k + n;
        uphill = 0;
      end
    end
    
  end    
  
  % display some information
  if opts.display, disp(sprintf('k: %d  Hf: %g  DHf: %g  DHs: %g  dx: %g', k, Hf, DHf, DHs, dx)); end
  
  % update the trace variable
  if nargout > 3
    trace = struct('k', [trace.k k], 'x', [trace.x x], 'f', [trace.f f], 'h', [trace.h norm(h, inf)], 'dx', [trace.dx dx], 'rho', [trace.rho rho], 'obj', [trace.obj Hf]); % trace variable
  end
  
  % test termination condition
  if k >= opts.kmax, stop = 2; end
  
end

if opts.display
  switch stop
  case 1
    disp('stop: step length too small')
  case 2
    disp('stop: reached the maximum allowed number of function evaluations')
  case 3
    disp('stop: trust region too small')
  case 4
    disp('stop: no decrease in local model')
  end
end
