function results = piston(k, t)
% Piston simulator
%
%    results = piston(k, t)
%
% k       : vector of reservoir permeabilities
% t       : vector of evaluation times
%
% results : a vector with  the producing well pressure at the times given in t

% js@imm.dtu.dk, November 19, 2002

Nblocks = length(k);
Ntimes = length(t);

Q = 0.92e-3;   % Q    - pump flow rate
L = 250.0;     % L    - reservoir length
H = 10.0;      % H    - reservoir height
W = 250.0;     % W    - reservoir width
phi = 0.3;     % phi  - porosity
muo = 1e-3;    % muo  - oil viscosity
muw = 3e-3;    % muw  - water viscosity
kro1 = 1.0;    % kro1 - oil relative perm.
krw1 = 0.3;    % krw1 - water relative perm.
pinj = 12.5;   % pinj - injection pressure

if 0    % 1: check input  0: no check
  if (Nblocks<=0) | (Ntimes>=100), error('PE - illegal Ntimes or Nblocks'); end
  i = (k <= 0);
  if ~isempty(find(i)), error(sprintf('PE - illegal block perm no. %d\nthe value was %g\n', i+10, t(10+i))), end
  i = (t < 0);
  if ~isempty(find(i)), error(sprintf('PE - illegal time value no. %d\nthe value was %g\n', i+10, t(10+i))), end
end

C1 = Q / (H * W * phi) * 86400;
C2 = L / Nblocks;
C3 = Q * muw / (krw1 * W * H) * C2 * 1e15;
C4 = Q * muo / (kro1 * W * H) * C2 * 1e15;
C5 = Q / (W * H) * C2 * 1e15;
C6 = muo / kro1;
C7 = muw / krw1;
C8 = (Q / (W * H))^2 / phi * (C7 - C6) * 86400 * 1e15;

results = zeros([Ntimes 1]);

for i = 1 : Ntimes
  xt = C1 * t(i);
  if xt >= L, Nf = Nblocks + 1; else Nf = floor(xt / C2) + 1; end
  pwf = 0;
  if Nf <= Nblocks
    pwf = pwf + C3 * sum(1 ./ k(1 : Nf - 1));
    pwf = pwf + C4 * sum(1 ./ k(Nf + 1 : Nblocks));
    pwf = pwf + C5 / k(Nf) * (Nf * C6 - (Nf - 1) * C7) + C8 / k(Nf) * t(i);
    pwf = pinj * 1e6 - pwf;
  else
    pwf = pwf + sum(1 ./ k(1 : Nblocks));
    pwf = pinj * 1e6 - pwf * C3;
  end
  results(i) = pwf * 1e-6;
end
