function A = broyunfac(A, x, xnew, f, fnew, eta)
%BROYUNFAC  Broyden's update, unfactored form
%
% Call
%       A = broyunfac(A, x, xnew, f, fnew, eta)
%
% where
%
%  A    : matrix to update
%  x    : current iterate
%  xnew : new/tentative iterate
%  f    : function values at x, f(x)
%  fnew : function values at xnew, f(xnew)
%  eta  : noise scale, if omitted default value is eps (minimum allowed value)
%
% Only rows of A with suggested updates larger than the estimated noise are updated.

% js@imm.dtu.dk, September 13, 2002

% Modelled after algorithm A8.3.1 in "A Modular System of Algorithms for Unconstrained Minimization and Nonlinear Equations" by Robert B. Schnabel,
% from the book: "Numerical Methods for Unconstrained Optimization and Nonlinear Equations" by J.E. Dennis, Jr. and Robert B. Schnabel

% set noise scale
if nargin < 6, eta = eps; else eta = max(eps, eta); end

% step
s = xnew - x;

% ||s||_2^2
denom = s' * s;

% avoid division by a very small number
if denom > eta

  % suggested row updates
  r =  fnew - f - A * s;

  % select indices of rows for which the suggested update
  % is larger than the estimated noise
  idx = find(abs(r) >= eta * (abs(fnew) + abs(f)));

  if ~isempty(idx)
  
    % apply updates to the selected rows
    A(idx, :) = A(idx, :) + r(idx) * (s / denom)';
  
  end
  
end
