function [A, b, alpha, Fr] = mapping_parameter_extraction(A, b, alpha, coarse, X, F, DF, HF, opts, varargin)

% Determine mapping parameters A, b, alpha for each of the mmap response functions
%
% Call
%       [A, b, alpha] = mapping_parameter_extraction(A, b, alpha, coarse, X, F, DF, HF, opts, varargin)
%
% opts.diag_a determines the number of mapping parameters
%   0 : a full input mapping parameter matrix A is used (n*n+n+1 mapping parameters)
%   1 : a diagonal input mapping parameter matrix A is used (2n+1 mapping parameters)
%
% k iterations have been performed in the main loop 
%   corresponding to k evaluations of the fine model
%
% Input
%
% A         : (n,n,mmap) matrix with the input mapping parameters A_i as the elements (:,:,i)
% b         : (n,mmap) matrix with the input mapping parameters b_i in the ith column
% alpha     : (1,mmap) vector with the output mapping parameters alpha_i as the ith element
% coarse:   : function handle to the coarse model 
% X         : (n,k) matrix with the x-iterates ( X(:,j) = x_j )
% F         : (m,k) matrix with the fine model vector evaluated at  the x-iterates
%             ( F(:,j) = f(x_j) )
% DF        : (m,n,k) matrix with the Jacobians of the fine model evaluated at the x-iterates
%             ( DF(:,:,j) = df(x_j) )
% HF        : (1,k) vector with the merit functions for the k fine model vectors
%             ( HF(1,j) = H(f(x_j)) )
% opts      : options from smopts
% P1, P2, ... : optional parameters passed to coarse model
%
% Output
%
% A         : (n,n,mmap) matrix with the updated input mapping parameters A_i as the elements (:,:,i)
% b         : (n,mmap) matrix with the updated input mapping parameters b_i in the ith column
% alpha     : (1,mmap) vector with the updated output mapping parameters alpha_i as the ith element
% Fr        : (mmap,1) vector with final values of the marquardt obj. fct. 0.5*r_i'*r_i for each of the mmap response functions 
%
% Pernille Brock, IMM, DTU, September 21, 2004

% determine diag_a flag and number of response functions and design parameters
diag_a = opts.diag_a;
m = size(F,1);
mmap = length(alpha);
n = size(X,1);

% initialize
Fr = zeros(mmap,1);

% set options for marquardt, see marquardt for further details
optsm = [1e-8 1e-4 1e-8 100 1e-12];

% find best iterate so far measured in merit function of the fine model
[HFbest index] = min(HF);
dfbest = DF(:,:,index);

% Update mapping parameters for the m response functions
for i = 1:mmap
    p0 = map(A(:,:,i), b(:,i), alpha(i), diag_a);     % mapping parameter vector
    [P, info, perf] = marquardt(@residual, p0, optsm, coarse, i, X, F, dfbest(i,:)', index, mmap, opts, varargin{:});
    p = P(:,end);
    [A(:,:,i), b(:,i) alpha(i)] = map(p, n, diag_a);
    Fr(i) = info(1);
    if opts.display > 1
        disp(sprintf('marquardt-info for mapping parameter extraction for response function %d',i))
        disp(info)
    end
end

%-------------------------------------------------------------------------------------------------

function [r, dr] = residual(p, coarse, i, X, F, df, index, mmap, opts, varargin)
    
% Calculates the residual between the fine model and the surrogate model
% for the ith response function.
% Call
%       [r, dr] = residual(p, coarse, i, X, F, df, index, opts, varargin)
%
% The norm of the residual is minimized in order to update the mapping parameters.
% k iterations have been performed in the main loop.
%
% The residual for the ith response function is given by
%   r_i(p) = [  s_i(x_1,p) - f_i(x_1)
%                          .
%                          .
%             s_i(x_k-1,p) - f_i(x_k-1)
%              ds_i(x_k,p) - df_i(x_k)  ]
%
% opts.diag_a determines the number of mapping parameters
%   0 : a full input mapping parameter matrix A is used (n*n + n + 1 mapping parameters)
%   1 : a diagonal input mapping parameter matrix A is used (2n + 1 mapping parameters)
% opts.weight_f determines the relative weight of the function value residual
%
% Input
%
% p     : the current parameter vector in which the residual is evaluated
% coarse: function handle to the coarse model
% i     : the response function number (integer in the range 1,...,mmap)
% X     : (n,k) matrix with the x-iterates ( X(:,j) = x_j )
% F     : (m,k) matrix with the fine model vector evaluated at the x-iterates
%         ( F(:,j) = f(x_j) )
% df    : (n,1) column vector with the gradient of the fine model's ith response
%         evaluated at the best x-iterate 
% index : the iteration index of the best iterate
% mmap  : number of response functions for mapping parameter extraction
% opts  : options from smopts
% P1, P2, ... : optional parameters passed to coarse model
%
% Output
%
% r     : the residual vector function (column vector with (k-1+n) elements)
% dr    : the Jacobian of the residual function wrt. the parameter vector p
%         (k-1+n, np) matrix
%
% Pernille Brock, IMM, DTU, September 21, 2004

% determine number of design parameters, iterations, mapping parameters and diag_a flag
n = size(X,1);
k = size(X,2);
np = length(p);
diag_a = opts.diag_a;

% determine relative step length used in fdgrad
eta = opts.eta;

% determine weighting factors for residual
fac_f = opts.weight_f;
fac_df = 1;

% reserve space for r, dr
r = zeros(k-1+n,1);
if nargout > 1
    dr = zeros(k-1+n, np);
end

% determine best iterate and function vector
xbest = X(:,index);
fbest = F(:,index);
if k > 1
    X = [X(:,1:index-1) X(:,index+1:end)];
    F = [F(:,1:index-1) F(:,index+1:end)];
else
    X = [];
    F = [];
end

% determine mapping parameters A, b and alpha
[A, b, alpha] = map(p, n, diag_a);

% calculate residual (and Jacobian wrt. p)
switch nargout
    case 1
        for j = 1:k-1
            s = surrogate_model_i(X(:,j), coarse, xbest, fbest, A, b, alpha, i, mmap, opts, varargin{:});
            r(j) = s - F(i,j);
        end
        [s, ds] = surrogate_model_i(xbest, coarse, xbest, fbest, A, b, alpha, i, mmap, opts, varargin{:});
        r(k:end) = ds - df;
    case 2
        for j = 1:k-1
            [s, ds, c, cbest, dc, dcbest] = surrogate_model_i(X(:,j), coarse, xbest, fbest, A, b, alpha, i, mmap, opts, varargin{:});
            r(j) = s - F(i,j);
            if diag_a == 1       % size(dr,2) = 2n + 1
                dr(j,1:n) = alpha*(dc'.*X(:,j)'-dcbest'.*xbest');
                dr(j,n+1:2*n+1) = [ alpha*(dc'-dcbest') (c-cbest) ];
            elseif diag_a == 0   % size(dr,2) = n*n + n + 1
                for h = 1:n
                    dr(j,n*(h-1)+1:h*n) = alpha*(dc'*X(h,j)-dcbest'*xbest(h));
                end
                dr(j,n*n+1:n*n+n+1) = [ alpha*(dc'-dcbest') (c-cbest) ];
            else
                error('diag_a must be 0 or 1')
            end
        end
        [s, ds] = surrogate_model_i(xbest, coarse, xbest, fbest, A, b, alpha, i, mmap, opts, varargin{:});
        r(k:end) = ds - df;
        [r(k:end), dr(k:end,:)] = fdgrad(@gradresidual, p, r(k:end), eta, df, coarse, xbest, fbest, i, mmap, opts, varargin{:});
    otherwise
        error('number of output arguments must be 1 or 2')
end

% normalization factors for residual
seps = sqrt(eps);
w_df = fac_df/(seps + norm(df,inf));
if k > 1
    w_f = fac_f/(seps + norm(F(i,:),inf));
end

% normalize residual
if k > 1
    r(1:k-1) = w_f*r(1:k-1);
end
r(k:end) = w_df*r(k:end);
if nargout > 1
    if k > 1
        dr(1:k-1,:) = w_f*dr(1:k-1,:);
    end
    dr(k:end,:) = w_df*dr(k:end,:);
end

%-------------------------------------------------------------------------------------------------

function rgrad = gradresidual(p, df, coarse, xbest, fbest, i, mmap, opts, varargin)

% calculates the gradient residual (the last n elements of the residual)
% used in the forward diff. appr. of dr(k:k-1+n,:)

% determine diag_a flag and number of design parameters
diag_a = opts.diag_a;
n = length(xbest);

% determine mapping parameters A, b and alpha
[A, b, alpha] = map(p, n, diag_a);

% evaluate surrogate model and return gradient residual
[s, ds] = surrogate_model_i(xbest, coarse, xbest, fbest, A, b, alpha, i, mmap, opts, varargin{:});
rgrad = ds - df;
