function z = parameter_extraction(x, coarse, xk, zk, B, zast, f, D, opts, varargin)
% Parameter extraction
%
%    z = parameter_extraction(x, coarse, xk, zk, B, zast, f, D, opts, P1, P2, ...)
%
% opts.petype determines parameter extraction type
% 1 : normal, min_z ||c(z) - f(x)||
% 2 : regularized wrt. z*,
%       min_z { (1-lambda) * ||c(z) - f(x)|| + lambda * ||z - z*|| }
% 3 : regularized wrt. x,
%       min_z { (1-lambda) * ||c(z) - f(x)|| + lambda * ||z - x|| }
% 4 : regularized wrt. gradients approximation,
%       min_z { (1-lambda) * ||c(z) - f(x)|| + lambda * ||c'(z) - D|| },
%     where D approx. f'(x)
%
% opts.lambda determines the value of lambda
%
% Input
%   x       point to be mapped
%   coarse  coarse model  function handle
%   xk      current iterate
%   zk      mapping of current iterate, p(xk)
%   B       current approximation to the Jacobian of p
%   zast    coarse model minimizer
%   f       fine model evaluated at x, f(x)
%   D       the Jacobian of f at x, or the current approximation to it
%   opts    options from smopts
%   P1,P2,... optional arguments passed to the coarse model

% js@imm.dtu.dk, January 26, 2003

% set the starting point for the nonlinear problem
% if not all necessary variables are supplied, default start by x
switch opts.pestart
case 1
  if ~isempty(zast)
    z = zast;
  else
    z = x;
  end
case 2
  z = x;
case 3
  if ~(isempty(B) | isempty(xk) | isempty(zk))
    z = B * (x - xk) + zk;
  else
    z = x;
  end
case 4
  if ~isempty(zk)
    z = zk;
  else
    z = x;
  end
otherwise, error(sprintf('No starting point number %d', opts.pstart))
end    

petype = opts.petype;

% check if we have all necessary variables, else default to usual space mapping
switch petype
case 2
  if isempty(zast), petype = 1; end
case 4
  if isempty(D), petype = 1; end
end

jacobian = 1;

% solve the parameter extraction problem
switch petype
case 1  
  % solve the usual parameter extraction problem
  bias = [];
case 2
  % solve the regularized parameter extraction problem, distance to zast
  if ~isempty(zast)
    bias = zast;
  else
    bias = x; 
  end
case 3
  % solve the regularized parameter extraction problem, distance to x
  bias = x;
case 4
  % solve the regularized parameter extraction problem using derivative information
  bias = D;
  jacobian = 0;
otherwise
  error(sprintf('petype = %d, undefined parameter extration type', opts.petype))
end

if jacobian
  % jacobian is available
  z = smdirect(2, @residual, z, [], [], 0, smopts(opts, 'display', 0, 'kmax', min(50, opts.kmax)), coarse, f, opts.lambda, bias, varargin{:});
else
  % use forward difference approximation to the Jacobian
  z = smdirect(2, @fdgradobj, z, [], [], 0, smopts(opts, 'display', 0, 'kmax', min(50, opts.kmax)), @residual, coarse, f, opts.lambda, bias, varargin{:});
end

function [e, de] = residual(z, func, y, lambda, bias, varargin)
% Residual function
%
%   [e, de] = residual(z, func, y, lambda, bias, P1, P2, ...)
%  

% js@imm.dtu.dk, January 26, 2003

if nargout > 1 | size(bias,2) > 1
  [c, de] = feval(func, z, varargin{:});
else
  c = feval(func, z, varargin{:});
end
e = c - y;
if ~isempty(bias)
  sqrtlambda = sqrt(lambda);
  sqrtlambda1 = sqrt(1 - lambda);
  if size(bias,2) == 1
    % regularize wrt. z* or x
    e = [sqrtlambda1 * e; sqrtlambda * (z - bias)];
    if nargout > 1
      de = [sqrtlambda1 * de; sqrtlambda * eye(length(z))];
    end
  else
    % regularize wrt. gradients
    e = [sqrtlambda1 * e; sqrtlambda * (de(:) - bias(:))];
    if nargout > 1
      error('Gradient of parameter extraction objective function not implemented for regularizing with gradients.')
    end
  end
end 
