function [x, l, Hl] = smtrlinear(H, D, xk, fk, dx, A, b, eq, opts)
% Solve the trust region problem for a linear model.
%
%  [x, l, Hl] = smtrlinear(H, D, xk, fk, dx, A, b, eq, opts)
%
%
%    min H(l(x))
%     x
%    s.t.   ||x - xk||_inf  <=  dx
%               A * x       <=  b    (first eq. rows are equality constraints)
%
%
%   where  l(x) = D * (x - xk) + fk
%

% js@imm.dtu.dk, January 20, 2003

n = length(xk);

switch H
case 'minimax'
  % optimize linear model, x = min_x max { D * (x - x_k) + f(x_k) }
  % subject to linear constraints (user and trust region constraints),
  
  % define trust region constraints, -dx <= h <= dx
  Atr = [[-eye(n); eye(n)] zeros(2*n, 1)];
  btr = repmat(dx, 2*n, 1);
  
  % setup user constraints
  if ~isempty(b)
    b = b - A * xk;
    [Ausr, busr, Aeq, beq] = split_constraints(A, b, eq);
    Ausrx = [Ausr zeros(size(busr))];
  else
    Ausrx = []; Aeq = [];
    busr = []; beq = [];
  end
    
  % solve  min h(n+1)  subject to:   D * h(1:n) + fk <= h(n+1), trust region and user constraints
  [h, Hl, exitval] = linprog([zeros(n, 1); 1], [[D repmat(-1, length(fk), 1)]; Atr; Ausrx], [-fk; btr; busr], Aeq, beq, [], [], [], optimset('LargeScale', 'off','Display', 'off'));
  if exitval < 0, error('solving of subproblem failed'); end
  
  h = h(1 : n);
  
  l = D * h + fk;
  
  x = xk + h;
case 2
  % optimize linear model, x = min_x { .5 || D * (x - x_k) + f(x_k) ||_2^2 }
  % subject to linear constraints (user and trust region constraints),
  
  % define trust region constraints, -dx <= h <= dx
  Atr = [-eye(n); eye(n)];
  btr = repmat(dx, 2*n, 1);
  
  % setup user constraints
  if ~isempty(b)
    b = b - A * xk;
    [Ausr, busr, Aeq, beq] = split_constraints(A, b, eq);
  else
    Ausr = []; Aeq = [];
    busr = []; beq = [];
  end
  
  DD = D' * D;
  if opts.display >= 2, disp(sprintf('cond(DD)=%g', cond(DD))); end
  cD = cond(DD);
  if cD ~= inf
    if cD >= 1/sqrt(eps)
      % condition number too large, apply the change suggested in Dennis & Schnabel p. 161
      DD = DD + sqrt(n * eps) * norm(DD, 1) * eye(n);
      if opts.display >= 2, disp(sprintf('corrected cond(DD)=%g', cond(DD))); end
    end
    fD = (fk' * D)';
    
    % solve  min { .5 h' * DD * h + fD' * h } subject to trust region and user constraints
    [h, fval, exitval] = quadprog(DD, fD, [Atr; Ausr], [btr; busr], Aeq, beq, [], [], [], optimset('Largescale', 'off', 'Display', 'off'));
    if exitval < 0, error('solving of subproblem failed'); end

    l = D * h + fk;
    x = xk + h;
  else
    % the derivatives are zero
    % set h to zero and exit
    
    h = zeros(size(xk));
    l = fk;
    x = xk;
  end
  Hl = smmerit(H, l);
  
otherwise
  if isnumeric(H), s = num2str(H); else s = H; end
  error(sprintf('direct optimization for the merit function "%s" is not implemented', s))
end
