function [s, ds] = surrogate_model(x, coarse, xbest, fbest, A, b, alpha, ivec, opts, varargin)

% Calculates the surrogate model vector and it's Jacobian 
% The interpolating surrogate model for the ith response is given by
%   s_i(x) = alpha_i*( c_i(A_i*x+b_i) - c_i(A_i*xbest+b_i) ) + fbest_i
%
% Call
%       [s, ds] = surrogate_model(x, coarse, xbest, fbest, A, b, alpha, ivec, opts, varargin)
% Input
%
% x         : the point in which the surrogate model is evaluated (n,1)
% coarse:   : function handle to the coarse model 
% xbest     : the best iterate so far (n,1) (the surrogate interpolates the fine
%             model exactly in this point)
% fbest     : the fine model response vector (m,1) evaluated at xbest
% A         : (n,n,mmap) matrix with the input mapping parameter matrix A_i as the elements (:,:,i)
% b         : (n,mmap) matrix with the input mapping parameter vector b_i in the ith column
% alpha     : (1,mmap) vector with the output mapping parameter alpha_i as the ith element
% ivec      : row vector [1:mmap] with the numbers of the response functions to be evaluated
% opts      : options from smopts
% P1, P2, ... : optional parameters passed to coarse model
%
% Output
%
% s         : the surrogate model vector (m,1) evaluated at x
% ds        : the Jacobian (m,n) of the surrogate model evaluated at x
%             (row number i contains the transposed gradient of the ith surrogate function)
%             for m =1, the gradient is returned
%
% Pernille Brock, IMM, DTU, September 21, 2004

% reserve space for s, ds
m = length(fbest);
mmap = length(ivec);
s = zeros(m,1);
if nargout > 1
    ds = zeros(m,length(x));
end

% the struct with parameters passed directly to the coarse model
S = varargin{:};

switch nargout
    case 1
        if norm(x-xbest) == 0        % the surrogate is identical to the fine model at xbest
            s(:) = fbest;
        else                         % x ~= xbest
            for i = ivec
                Si = changeS(S, i);  % change the state/target variable vector to the ith state/target variable (if present)
                c = feval(coarse, (A(:,:,i)*x+b(:,i)), Si);
                cbest = feval(coarse, (A(:,:,i)*xbest+b(:,i)), Si);
                if ~isfield(S, 'state') & ~isfield(S, 'target')
                    c = c(i); cbest = cbest(i);
                elseif m ~=mmap
                    c = c(1); cbest = cbest(1);
                end
                s(i) = alpha(i)*(c-cbest) + fbest(i);
            end
            if m ~= mmap, s(mmap+1:end) = -s(1:mmap); end 
        end
    case 2
        if norm(x-xbest) == 0        % the surrogate is identical to the fine model at xbest
            s(:) = fbest;
            for i = ivec
                Si = changeS(S, i);  % change the state/target variable vector to the ith state/target variable (if present)
                [c, dc] = feval(coarse, (A(:,:,i)*x+b(:,i)), Si);
                if ~isfield(S, 'state') & ~isfield(S, 'target')
                    c = c(i); dc = dc(i,:)';
                elseif m ~=mmap
                    c = c(1); dc = dc(1,:)';    
                end
                ds(i,:) = alpha(i)*dc'*A(:,:,i);
            end
        else                         % x ~= xbest
            for i = ivec
                Si = changeS(S, i);  % change the state/target variable vector to the ith state/target variable (if present)
                [c, dc] = feval(coarse, (A(:,:,i)*x+b(:,i)), Si);
                cbest = feval(coarse, (A(:,:,i)*xbest+b(:,i)), Si);
                if ~isfield(S, 'state') & ~isfield(S, 'target')
                    c = c(i); cbest = cbest(i); dc = dc(i,:)';
                elseif m ~=mmap
                    c = c(1); cbest = cbest(1); dc = dc(1,:)';    
                end
                s(i) = alpha(i)*(c-cbest) + fbest(i);
                ds(i,:) = alpha(i)*dc'*A(:,:,i);
            end
            if m ~= mmap, s(mmap+1:end) = -s(1:mmap); end 
        end
        if m ~= mmap, ds(mmap+1:end,:) = -ds(1:mmap,:); end
    otherwise
        error('wrong number of output arguments')
end

% return gradient as a column vector
if nargout == 2 & m == 1
    ds = ds';
end

%--------------------------------------------------------------------------

function Si = changeS(S, i)

% changes the struct with parameters passed directly to the coarse model
% if present, the state variable vector is replaced by the ith element of the vector
% if present, the target variable vector is replaced by the ith element of the vector
% when passing Si as argument to the coarse model, it is only evaluated at the ith state/target variable, 
%   ie the coarse model returns c(i) and dc(i,:)'
%
% Pernille Brock, IMM, DTU, September 21, 2004

Si = S;
if isstruct(S)
    if isfield(S, 'state'), Si.state = S.state(i); end
    if isfield(S, 'target'), Si.target = S.target(i); end
end
