function [s, ds, c, cbest, dc, dcbest] = surrogate_model_i(x, coarse, xbest, fbest, A, b, alpha, i, mmap, opts, varargin)

% Calculates the ith surrogate model element and it's gradient
% The interpolating surrogate model for the ith response is given by
%   s_i(x) = alpha_i*( c_i(A_i*x+b_i) - c_i(A_i*xbest+b_i) ) + f_i(x_best)
%
% Call
%       [s, ds, c, cbest, dc, dcbest] = surrogate_model_i(x, coarse, xbest, fbest, A, b, alpha, i, mmap, opts, varargin)
%
% Input
%
% x         : the point in which the surrogate model is evaluated (n,1)
% coarse:   : function handle to the coarse model 
% xbest     : the best iterate so far (n,1) (the surrogate interpolates the fine
%             model exactly in this point)
% fbest     : the fine model response vector (m,1) evaluated at xbest
% A         : (n,n) matrix, the input mapping parameter matrix for the ith response
% b         : (n,1) vector, the input mapping parameter vector for the ith response 
% alpha     : scalar, the output mapping parameter alpha for the ith response
% i         : the number of the response function to be evaluated
% mmap      : number of response functions for mapping parameter extraction
% opts      : options from smopts
% P1, P2, ... : optional parameters passed to coarse model
%
% Output
%
% s         : the ith element of the surrogate model evaluated at x
% ds        : the gradient (n,1) of the ith element of the surrogate model
% c         : the ith element of the coarse model at x (c_i(x))
% dc        : the gradient (n,1) of the ith element of the coarse model at x 
% cbest     : the ith element of the coarse model at xbest (c_i(xbest))
% dcbest    : the gradient (n,1) of the ith element of the coarse model at xbest
%
% Pernille Brock, IMM, DTU, September 21, 2004

% determine number of response functions
m = length(fbest);

% reserve space for s, ds
s = 0;
if nargout == 2
    ds = zeros(1,length(x));
elseif nargout == 6
    ds = [];
end

% determine the struct with parameters passed directly to the coarse model
S = varargin{:};

switch nargout
    case 1
        if norm(x-xbest) == 0           % the surrogate is identical to the fine model at xbest
            s = fbest(i);
        else                            % x ~= xbest
            Si = changeS(S, i);         % change the state/target variable vector to the ith state/target variable (if present)
            c = feval(coarse, (A*x+b), Si);
            cbest = feval(coarse, (A*xbest+b), Si);
            if ~isfield(S, 'state') & ~isfield(S, 'target')
                c = c(i); cbest = cbest(i);
            elseif m ~=mmap
                c = c(1); cbest = cbest(1);
            end
            s = alpha*(c-cbest) + fbest(i);
        end
    case 2
        if norm(x-xbest) == 0           % the surrogate is identical to the fine model at xbest
            s = fbest(i);
            Si = changeS(S, i);         % change the state/target variable vector to the ith state/target variable (if present)
            [c, dc] = feval(coarse, (A*x+b), Si);
            if ~isfield(S, 'state') & ~isfield(S, 'target')
                c = c(i); dc = dc(i,:)';
            elseif m ~=mmap
                c = c(1); dc = dc(1,:)';
            end
            ds(1,:) = alpha*dc'*A;
        else                            % x ~= xbest
            Si = changeS(S, i);         % change the state/target variable vector to the ith state/target variable (if present)
            [c, dc] = feval(coarse, (A*x+b), Si);
            cbest = feval(coarse, (A*xbest+b), Si);
            if ~isfield(S, 'state') & ~isfield(S, 'target')
                c = c(i); cbest = cbest(i); dc = dc(i,:)';
            elseif m ~=mmap
                c = c(1); cbest = cbest(1); dc = dc(1,:)';
            end
            s = alpha*(c-cbest) + fbest(i);
            ds(1,:) = alpha*dc'*A;
        end
    case 6
        if norm(x-xbest) == 0           % the surrogate is identical to the fine model at xbest
            s = fbest(i);
            Si = changeS(S, i);         % change the state/target variable vector to the ith state/target variable (if present)
            [c, dc] = feval(coarse, (A*x+b), Si);
            if ~isfield(S, 'state') & ~isfield(S, 'target')
                c = c(i); dc = dc(i,:)';
            elseif m ~=mmap
                cc = c(1); dc = dc(1,:)';
            end
            cbest = c; dcbest = dc;
        else                            % x ~= xbest
            Si = changeS(S, i);         % change the state/target variable vector to the ith state/target variable (if present)
            [c, dc] = feval(coarse, (A*x+b), Si);
            [cbest, dcbest] = feval(coarse, (A*xbest+b), Si);
            if ~isfield(S, 'state') & ~isfield(S, 'target')
                c = c(i); cbest = cbest(i); dc = dc(i,:)'; dcbest = dcbest(i,:)';
            elseif m ~=mmap
                c = c(1); cbest = cbest(1); dc = dc(1,:)'; dcbest = dcbest(1,:)';
            end
            s = alpha*(c-cbest) + fbest(i);
        end
    otherwise
        error('wrong number of output arguments')
end

% return gradient as a column vector
if nargout == 2
    ds = ds';
end

%--------------------------------------------------------------------------

function Si = changeS(S, i)

% changes the struct with parameters passed directly to the coarse model
% if present, the state variable vector is replaced by the ith element of the vector
% if present, the target variable vector is replaced by the ith element of the vector
% when passing Si as argument to the coarse model, it is only evaluated at the ith state/target variable, 
%   ie the coarse model returns c(i) and dc(i,:)'
%
% Pernille Brock, IMM, DTU, September 21, 2004

Si = S;
if isstruct(S)
    if isfield(S, 'state'), Si.state = S.state(i); end
    if isfield(S, 'target'), Si.target = S.target(i); end
end
