function [xk, fk, Hfk, stop, trace] = smho(H, fine, coarse, x0, A, b, eq, opts, varargin)
% Hybrid space mapping algorithm with orthogonal steps
%
% Call
%    [xk, fk, Hfk, stop, trace] = smho(H, fine, coarse, x0, A, b, eq, opts, P1, P2, ...)
%
% Input
%
% H       : merit function for vector functions
% fine    : function handle to the fine model
% coarse  : function handle to the coarse model
%           the coarse model must provide both a response vector and
%           the Jacobian matrix when evaluated
% x0      : starting point
% A, b    : linear constraints on fine model parameters
%           A * x <= b
% eq      : number of (the first) rows of the linear constraints
%           which are equality constraints
% opts    : options from smopts, if left empty the default values are used
% P1, P2, ... : optional parameters passed to fine and coarse model
%
%
% Output
%
% xk      : solution reached by the algorithm
% fk      : the fine model response at the solution
% Hfk     : H(fk)
% stop    : reason for stopping the algorithm
%              1  step length too small
%              2  reached the maximum allowed number of function evaluations
%              3  trust region too small
%              4  no decrease in local model
% trace   : trace of the iteration process

% js@imm.dtu.dk, January 27, 2003

% check if options are provided
if nargin < 8, opts = []; end

% otherwise get them from the default values
if isempty(opts), opts = smopts; end

if nargin < 7, eq = 0; end
if nargin < 6, A = []; b = []; end
if nargin < 4, error('No enough input arguments. At least four input arguments should be supplied.'); end

% determine the number of variables
n = size(x0, 1);

% determines if next step is a space mapping step otherwise it is an orthogonal step
smstep = 1;
                  
% determines if the algorithm is in stage 2 (using only the linear model)
linear = 0;

% Jacobian matrix approximation to the mapping function
% initially a unity mapping is assumed
B = eye(n);       

% Jacobian matrix approximation to the fine model
% (here the variable is left empty, it will be initialized later on)
D = [];

% matrix for storing previous steps
E = [];

% set the initial size of the trust region
dx = opts.dx;
dxmax = 1e3 * (1 + max(dx, norm(x0)));  % max trust region size

% success indicator (the algorithm continues while stop is false (zero))
stop = 0;         

% fine model fct. evaluation counter
k = 0;

% optimize coarse model
[zast, cast, Hcast] = smdirect(H, coarse, x0, A, b, eq, smopts(opts, 'display', 0), varargin{:});

% we set xk = zast (= z*) and use this as
% starting point for the space mapping algorithm
x = zast;

% evaluate fine model at x
if opts.display > 1
  disp('evaluating fine model')
end
f = feval(fine, x, varargin{:});
Hf = smmerit(H, f);
k = k + 1;

% parameter extraction
if opts.display > 1
  disp('parameter extraction')
end
z = parameter_extraction(x, coarse, [], [], B, zast, f, D, smopts(opts, 'display', 0), varargin{:});

if opts.initd == 1
  % initialize fine model Jacobian approximation, D = c'(z)
  [c D] = feval(coarse, z, varargin{:});
end

% define current iterate before entering the main loop
xk = x;
xn = 1 + norm(x);
fk = f;
Hfk = Hf;
zk = z;

% display some progress information
if opts.display, disp(sprintf('k: %d  Hf: %g', k, Hf)); end

if nargout > 3
  trace = struct('k', k, 'x', x, 'z', z, 'f', f, 'h', norm(x-x0, inf), 'dx', dx, 'w', ~linear, 'rho', [], 'obj', Hf); % trace variable
end

if k >= opts.kmax
  stop = 2;
end

%%% main loop %%%

rho = 0.0;  h = zeros(size(x)); w_not_reduced = 0;

while ~stop

  if smstep | linear  % take a space mapping or linear (stage 2) step
  
    if ~linear
      % stage 1, optimize non-linear surrogate

      % optimize mapped_model, xnew = arg min_x max{c(p_k(x))}
      % subject to linear constraints (user and trust region constraints),
  
      % define trust region constraints
      % xk-dx <= x <= xk+dx
      Atr = [-eye(n); eye(n)];
      btr = [-(xk-dx); xk+dx];
  
      if opts.display >= 2
        disp('Solving min_x H(s_k(x))')
      end
  
      % call direct optimization
      [x, s, Hs] = smdirect(H, @mapped_model, xk, [A;Atr], [b;btr], eq, smopts(opts, 'display', 0,'dx', max(dx/3, 2*opts.epsilon)), ...  % arguments for smdirect
        coarse, xk, zk, B, varargin{:}); % this line is arguments for mapped_model
  
      % the tentative step
      h = x - xk;
      
    else
      % stage 2, use only the linear model
    
      % optimize linear model subject to linear constraints
      % (user and trust region constraints)

      if opts.display >= 2
        disp('Solving min_x H( D * (x - x_k) + f(x_k) )')
      end
    
      [x, s, Hs] = smtrlinear(H, D, xk, fk, dx, A, b, eq, opts);
      
      h = x - xk;
      
    end

    % check if step length is too small
    if norm(h) <= opts.epsilon * xn
      stop = 1;
      break  % break the loop now
    end
    
  else
    % take an orthogonal step

    if opts.display >= 2
      disp('Orthogonal step')
    end
      
    switch opts.ortho_meth
    case 1
      
      % method described in FP's master thesis
    
      % calculate h orthogonal to previous steps
      h = opts.ortho_scale * calc_ortho(E);
      
      % scale h
      switch opts.ortho_scale_type
      case 1
        h = xn * h;
      case 2
        h = dx * h;
      otherwise, error(sprintf('No scaling type %d', opts.ortho_scale_type));
      end
    
      x1 = xk + h;   % forward step
      x2 = xk - h;   % backward step

      % mapped model prediction of the two steps
      par = struct('B', B, 'xk', x, 'zk', z, 'coarse', coarse, 'varargin', {varargin});
      s1 = feval(@mapped_model, x1, coarse, xk, zk, B, varargin{:});
      s2 = feval(@mapped_model, x2, coarse, xk, zk, B, varargin{:});
    
      % objective function values
      Hs1 = smmerit(H, s1);
      Hs2 = smmerit(H, s2);
    
      % is the forward step better (predicted value) than the backward step?
      % choose the better step
      if Hs1 <= Hs2
        x = x1;
        s = s1;
        Hs = Hs1;
      else
        h = -h;
        x = x2;
        s = s2;
        Hs = s2;
      end
      
    case 2
      
      % method of minimizing mapped coarse model approximation, x = arg min_x max { coarse(p_k(x)) }
      % s.t. linear constraints ensuring orthogonality

      switch opts.ortho_scale_type
      case 1
        dx_ort = opts.ortho_scale * xn;
      case 2
        dx_ort = opts.ortho_scale * dx;
      otherwise, error(sprintf('No scaling type %d', opts.ortho_scale_type));
      end
      
      % define orthogonal trust region constraints, x-dx_ort <= x <= x+dx_ort
      Atr = [-eye(n); eye(n)];
      btr = [-(xk-dx_ort); xk+dx_ort];
    
      % define orthogonality constraints
      Aort = myorth(E)';
      bort = -A_ort * xk;
      
      % number of orthogonality and user equality constraints
      leq_ort = leq + length(bort);
      
      % call direct optimization
      [x, s, Hs] = smdirect(H, @mapped_model, xk, [Aort;A;Atr], [bort;b;btr], leq_ort, smopts(opts, 'display', 0,'dx', max(dx/3, 2*opts.epsilon)), ...  % arguments for smdirect
        coarse, xk, zk, B, varargin{:}); % this line is arguments for mapped_model
    
      h = x - xk;

    end

    % check if step length is too small
    if norm(h) <= opts.epsilon * xn
      error('Orthogonal step too small!')
    end
  end
    
  % DHs = -Delta S_k(x) (= H(f(x_k)) - H(s_k(x_k + h)))
  DHs = Hfk - Hs;
  
  if (smstep | linear) & (DHs <= 0)
    
    % step is not accepted due to insufficient decrease in mapped model
    
    if linear
      stop = 4;  % no decrease in local model
      break
    end
    
    if opts.display > 1, disp('no decrease in local model'); end
    
    accept = 0;  % step is not accepted
    
  else
    
    if ~linear    
      % store the step
      E = [E h];
    end

    % evaluate fine model at x
    if opts.display > 1
      disp('evaluating fine model')
    end
    f = feval(fine, x, varargin{:});
    Hf = smmerit(H, f);
    k = k + 1;
    
    % parameter extraction
    if opts.display > 1
      disp('parameter extraction')
    end
    z = parameter_extraction(x, coarse, xk, zk, B, zast, f, D, smopts(opts, 'display', 0), varargin{:});
    
    % DHf = H(f(x_k)) - H(f(x_k + h))
    DHf = Hfk - Hf;
    
    % boolean determining if the tentative step is accepted
    accept = (DHf >= opts.delta1 * DHs);

    if smstep | linear % adjust trust region if step is not an orthogonal step

      % Update trust region
    
      % calculate gain
      rho = DHf / DHs;

      % set new trust region size
      if rho <= opts.delta2
        dx = opts.k1 * dx;
        % check if trust region is smaller than minimum allowed step length
        if dx <= opts.epsilon * xn
          stop = 3;  % stop before next iteration
        end
      end
      if rho >= opts.delta3, dx = min(dxmax, opts.k2 * dx); end
      
    end
    
    % Broyden update of mapping approximation
    Bnew = broyunfac(B, xk, x, zk, z);
    if opts.display > 2, disp(sprintf('||B-Bnew|| = %g', norm(B-Bnew, 'fro'))); end
    B = Bnew;
    
    if linear | (opts.initd == 1)
      % Broyden update of the linear approximation of the fine model
      Dnew = broyunfac(D, xk, x, fk, f);
      if opts.display > 2, disp(sprintf('||D-Dnew|| = %g', norm(D-Dnew, 'fro'))); end
      D = Dnew;
    end
    
  end
  
  if accept
    % accept the tentative iterate
    xk = x;
    xn = 1 + norm(x);
    fk = f;
    Hfk = Hf;
    zk = z;
    
    if ~linear
      
      smstep = 1; % take sm step in next iteration
      
      if k > n
        % we have exceeded n iterations
        
        % count the number of iterations we have not 'reduced w'
        w_not_reduced = w_not_reduced + 1;
        
        % if we have exceeded max number of allowed iterations without using
        % the linear model, switch to the linear model (stage 2)
        if w_not_reduced >= max(n, opts.max_w_not_reduced)
          
          linear = 1
          pause
        end
        
      end
      
    end

    if opts.display > 1, disp('step accepted'); end
    
  else
    
    if opts.display > 1, disp('step not accepted'); end
    
    if k <= n | ~smstep
      
      % stage 1
      % take a sm step in next iteration, if step in this iteration
      % was an orthogonal step, and vice versa
      % (last stage 1 step is always a sm-step)
      smstep = ~smstep;
      
    elseif ~linear

      % stage 1
      % k > n, goto stage two
      linear = 1;
      
      % initialize D according to strategy in defined in initd
      switch opts.initd
      case 1, D = D;
      case 2,
        % initialize fine model Jacobian approximation, D = c'(z_k) * B
        [ck D] = feval(coarse, zk, varargin{:});
      case 3
        if opts.display > 2, disp('finite difference'); end
        [f D] = fdgrad(fine, xk, fk, 0, varargin{:});
        k = k + n;
      otherwise, error(sprintf('initd should be either 1, 2 or 3, now initd = %d', opts.initd))
      end

      uphill = 0;
    
    else
      
      if opts.dofinitediff
        uphill = uphill + 1;
        if uphill >= opts.maxuphill & opts.maxuphill > 0
          if opts.display > 2, disp('finite difference'); end
          [f D] = fdgrad(fine, xk, fk, 0, varargin{:});
          k = k + n;
          uphill = 0;
        end
      end      
    end    
  end    
    
  % display some information
  if opts.display, disp(sprintf('k: %d  Hf: %g  DHf: %g  DHs: %g  dx: %g', k, Hf, DHf, DHs, dx)); end
  
  % update the trace variable
  if nargout > 3
    trace = struct('k', [trace.k k], 'x', [trace.x x], 'z', [trace.z z], 'f', [trace.f f], 'h', [trace.h norm(h, inf)], 'dx', [trace.dx dx], 'w', [trace.w ~linear], 'rho', [trace.rho rho], 'obj', [trace.obj Hf]); % trace variable
  end

  % test termination condition
  if k >= opts.kmax, stop = 2; end
  
end

if opts.display
  switch stop
  case 1
    disp('stop: step length too small')
  case 2
    disp('stop: reached the maximum allowed number of function evaluations')
  case 3
    disp('stop: trust region too small')
  case 4
    disp('stop: no decrease in local model')
  case 6
    disp('stop: solving of subproblem failed')
  end
  if ~linear
    disp('transition to linear model not complete')
  end
end


function ho = calc_ortho(H)
% Calculate a vector orthogonal to the columns of H.
% Adopted from work by Frank Pedersen, IMM, DTU, September 2001,
% modified to use coordinate directions instead of random vectors.

n = size(H, 1);
E = eye(n);
Q = myorth(H);
stop = 0;
for i = 1:n
  h = E(:, i);
  ho = h - Q*(Q'*h);
  nho = norm(ho);
  stop = nho > 1e-2 * norm(h);
  if stop, break; end
end
ho = ho / nho;

function  Q = myorth(A)
% Gram-Schmidt orthonormalization.
% Adopted from work by Frank Pedersen, IMM, DTU, September 2001.
% Original author: Hans Bruun Nielsen, IMM, DTU

[m n] = size(A);   Q = zeros(m, n);
ns = sum(A.^2);   ns0 = sqrt(ns);   p = 1 : n;
for  j = 1 : n
  [mn i] = max(ns(p(j:n)));
  jj = p(j - 1 + i);   p(j - 1 + i) = p(j);   p(j) = jj;
  w = A(:, jj);   nw = norm(w);
  if  nw < 10 * eps * ns0(jj), Q(:, jj) = zeros(m, 1);
  else
    w = w / nw;   Q(:, jj) = w;
    for  k = j+1 : n
      kk = p(k);   f = w' * A(:, kk);  
      A(:, kk) = A(:, kk) - f * w;
      ns(kk) = ns(kk) - f^2;
    end
  end
end
