function [xk, fk, Hfk, stop, trace] = smis(H, mmap, fine, coarse, x0, A, b, eq, opts, varargin)
% Space Mapping algorithm with Interpolating Surrogate model
%
% Call
%    [xk, fk, Hfk, stop, trace] = smis(H, mmap, fine, coarse, x0, A, b, eq, opts, P1, P2, ...)
%
% Input
%
% H       : merit function for vector functions
% mmap    : number of response functions for mapping parameter extraction
% fine    : function handle to the fine model
%           the fine model must provide both a response vector and
%           the Jacobian matrix when evaluated
% coarse  : function handle to the coarse model
%           the coarse model must provide both a response vector and
%           the Jacobian matrix when evaluated
% x0      : starting point
% A, b    : linear constraints on fine model parameters
%           A * x <= b
% eq      : number of (the first) rows of the linear constraints
%           which are equality constraints
% opts    : options from smopts, if left empty the default values are used
% P1, P2, ... : optional parameters passed to fine and coarse model
%
%
% Output
%
% xk      : solution reached by the algorithm
% fk      : the fine model response at the solution
% Hfk     : H(fk)
% stop    : reason for stopping the algorithm
%              1  step length too small
%              2  reached the maximum allowed number of function evaluations
%              3  trust region too small
%              4  no decrease in surrogate model
%              5  no decrease in fine model
% trace   : trace of the iteration process
%
% Pernille Brock, September 3, 2004

% check if options are provided
if nargin < 9, opts = []; end

% otherwise get them from the default values
if isempty(opts), opts = smopts; end

if nargin < 8, eq = 0; end
if nargin < 7, A = []; b = []; end
if nargin < 5, error('Not enough input arguments. At least four input arguments should be supplied.'); end

% determine the number of variables
n = size(x0, 1);

% set the initial size of the trust region
dx = opts.dx;
dxmax = 1e3 * (1 + max(dx, norm(x0)));  % max trust region size

% success indicator (the algorithm continues while stop is false (zero))
stop = 0;

% fine model fct. evaluation counter
k = 0;

% optimize coarse model
[zast, cast, Hcast] = smdirect(H, coarse, x0, A, b, eq, smopts(opts, 'display', 0), varargin{:});

% we set xk = zast (= z*) and use this as
% starting point for the space mapping algorithm
x = zast;

% evaluate fine model and Jacobian at x
if opts.display > 1
    disp('evaluating fine model')
end
[f, df] = feval(fine, x, varargin{:});
Hf = smmerit(H, f);
k = k + 1;

% define the trace variable
if nargout > 4
    trace = struct('k', k, 'x', x, 'f', f, 'df', df, 'h', norm(x-x0, inf), 'dx', dx, 'am', [], 'bm', [], 'alpham', [], ...
        'rho', [], 'obj', Hf, 'fr', []);  % trace variable
else
    trace = struct('x', x, 'f', f, 'df', df, 'obj', Hf); % trace variable
end

% initialize the mapping parameters A, b and alpha for each of the mmap response functions
Am = repmat(eye(n), [1, 1, mmap]);
bm = repmat(0, n, mmap);
alpham = repmat(1, 1, mmap);

% mapping parameter extraction
if opts.display > 1
    disp('mapping parameter extraction')
end
[Am, bm, alpham, Fr] = mapping_parameter_extraction(Am, bm, alpham, coarse, trace.x, trace.f, trace.df, trace.obj, opts, varargin{:});

if nargout > 4
    trace.am = Am; trace.bm = bm; trace.alpham = alpham; trace.fr = Fr; % update trace variable
end

% define current iterate before entering the main loop
xk = x;
xn = 1 + norm(x);       % NB check stopping criteria
fk = f;
Hfk = Hf;

% display some progress information
if opts.display, disp(sprintf('k: %d  Hf: %g', k, Hf)); end

% check stopping criteria
if k >= opts.kmax
  stop = 2;
end

%%% main loop %%%

rho = 0.0;  h = zeros(size(x));

while ~stop
      
    % stage 1, optimize non-linear surrogate
    
    % optimize surrogate_model, xnew = arg min_x H(s_k(x,p))
    % subject to linear constraints (user and trust region constraints),
    
    % define trust region constraints
    % xk-dx <= x <= xk+dx
    Atr = [-eye(n); eye(n)];
    btr = [-(xk-dx); xk+dx];
    
    % determine the best iterate and fine model function vector so far
    [Hfbest, index] = min(trace.obj);
    xbest = trace.x(:,index);
    fbest = trace.f(:,index);
    
    % display some information
    if opts.display >= 2
        disp('Best value of objective function before surrogate optimization')
        disp(sprintf('Hfbest = %g', Hfbest))
        disp('Solving min_x H(s_k(x))')
    end
    
    % call direct optimization
    [x, s, Hs] = smdirect(H, @surrogate_model, xk, [A;Atr], [b;btr], eq, ...
        smopts(opts, 'display', 0,'dx', max(dx/3, 2*opts.epsilon)), ...     % first two lines are arguments for smdirect
        coarse, xbest, fbest, Am(:,:,:,end), bm(:,:,end), alpham(:,:,end), [1:mmap], opts, varargin{:});    % this line is argument for surrogate_model
    
    % the tentative step
    h = x - xk;
    
    % check if step length is too small
    if norm(h) <= opts.epsilon * xn
        stop = 1;
        break  % break the loop now
    end
  
    % DHs = -Delta S_k(x) (= H(f(x_k)) - H(s_k(x_k + h)))
    DHs = Hfk - Hs;
  
    if DHs <= 0
    
        % step is not accepted due to insufficient decrease in combined model
    
        if opts.display > 1, disp('no decrease in surrogate model'); end
    
        stop = 4; break;

    end
    
    % evaluate fine model and Jacobian at x
    if opts.display > 1
        disp('evaluating fine model')
    end
    [f, df(:,:,k+1)] = feval(fine, x, varargin{:});
    Hf = smmerit(H, f);
    k = k + 1;
    
    % DHf = H(f(x_k)) - H(f(x_k + h))
    DHf = Hfk - Hf;
    
    % boolean determining if the tentative step is accepted
    accept = ( DHf > opts.epsilon*(1 + abs(Hfk)) );
    
    if accept
        % accept the tentative iterate
        xk = x;
        xn = 1 + norm(x);
        fk = f;
        Hfk = Hf;
    
        if opts.display > 1, disp('step accepted'); end
    else
        % step not accepted due to insufficient decrease in fine model
        if opts.display > 1, disp('no decrease in fine model'); end
        stop = 5; break;
    end
    
    % calculate gain
    rho = DHf / DHs;
    
    % Update trust region
    if rho > 0.5
        dx = min( [max(2*norm(h),dx) dxmax] );
    elseif rho < 1e-4
        dx = dx/3;
    end
    
    % check if trust region is smaller than minimum allowed step length
    if dx <= opts.epsilon * xn
        stop = 3;  % stop before next iteration
    end
  
    % display some information
    if opts.display, disp(sprintf('k: %d  Hf: %g  DHf: %g  DHs: %g  dx: %g', k, Hf, DHf, DHs, dx)); end
  
    % update the trace variable before mapping parameter extraction
    if nargout > 4
        trace = struct('k', [trace.k k], 'x', [trace.x x], 'f', [trace.f f], 'df', df, ...
            'h', [trace.h norm(h, inf)], 'dx', [trace.dx dx], 'am', Am, 'bm', bm, 'alpham', alpham, ...
            'rho', [trace.rho rho], 'obj', [trace.obj Hf], 'fr', Fr); % trace variable
    else
        trace = struct('x', [trace.x x], 'f', [trace.f f], 'df', df, 'obj', [trace.obj Hf]); % trace variable    
    end
    
    % mapping parameter extraction
    if opts.display > 1
        disp('mapping parameter extraction')
    end
    if nargout > 4
        [Am(:,:,:,k), bm(:,:,k), alpham(:,:,k), Fr(:,k)] = mapping_parameter_extraction(Am(:,:,:,k-1), bm(:,:,k-1), alpham(:,:,k-1), ...
            coarse, trace.x, trace.f, trace.df, trace.obj, opts, varargin{:});
    else
        [Am, bm, alpham] = mapping_parameter_extraction(Am, bm, alpham, coarse, trace.x, trace.f, trace.df, trace.obj, opts, varargin{:});
    end
    
    % update the trace variables Am, bm and alpham
    if nargout > 4
        trace.am = Am; trace.bm = bm; trace.alpham = alpham; trace.fr = Fr; % update trace variable
    end
    
    % test termination condition
    if k >= opts.kmax, stop = 2; end
    
end

% display stop
if opts.display
  switch stop
  case 1
    disp('stop: step length too small')
  case 2
    disp('stop: reached the maximum allowed number of function evaluations')
  case 3
    disp('stop: trust region too small')
  case 4
    disp('stop: no decrease in surrogate model')
  case 5
    disp('stop: no decrease in fine model')
  end
end

trace.stop = stop;
