function [xk, fk, Hfk, stop, trace] = smon(H, fine, coarse, x0, A, b, eq, opts, varargin)
% New space mapping formulation, with mapped coarse model
%
% Call
%    [xk, fk, Hfk, stop, trace] = smon(H, fine, coarse, x0, A, b, eq, opts, P1, P2, ...)
%
% Input
%
% H       : merit function for vector functions
% fine    : function handle to the fine model
% coarse  : function handle to the coarse model
%           the coarse model must provide both a response vector and
%           the Jacobian matrix when evaluated
% x0      : starting point
% A, b    : linear constraints on fine model parameters
%           A * x <= b
% eq      : number of (the first) rows of the linear constraints
%           which are equality constraints
% opts    : options from smopts, if left empty the default values are used
% P1, P2, ... : optional parameters passed to fine and coarse model
%
%
% Output
%
% xk      : solution reached by the algorithm
% fk      : the fine model response at the solution
% Hfk     : H(fk)
% stop    : reason for stopping the algorithm
%              1  step length too small
%              2  reached the maximum allowed number of function evaluations
%              3  trust region too small
%              4  no decrease in local model
% trace   : trace of the iteration process

% js@imm.dtu.dk, January 27, 2003

% check if options are provided
if nargin < 8, opts = []; end

% otherwise get them from the default values
if isempty(opts), opts = smopts; end

if nargin < 7, eq = 0; end
if nargin < 6, A = []; b = []; end
if nargin < 4, error('No enough input arguments. At least four input arguments should be supplied.'); end

% determine the number of variables
n = size(x0, 1);

% Jacobian matrix approximation to the mapping function
% initially a unity mapping is assumed
B = eye(n);       

% set the initial size of the trust region
dx = opts.dx;
dxmax = 1e3 * (1 + max(dx, norm(x0)));  % max trust region size

% success indicator (the algorithm continues while stop is false (zero))
stop = 0;         

% fine model fct. evaluation counter
k = 0;

% optimize coarse model
[zast, cast, Hcast] = smdirect(H, coarse, x0, A, b, eq, smopts(opts, 'display', 0), varargin{:});

% we set xk = zast (= z*) and use this as
% starting point for the space mapping algorithm
x = zast;

% evaluate fine model at x
if opts.display > 1
  disp('evaluating fine model')
end
f = feval(fine, x, varargin{:});
Hf = smmerit(H, f);
k = k + 1;

% parameter extraction
if opts.display > 1
  disp('parameter extraction')
end
z = parameter_extraction(x, coarse, [], [], B, zast, f, [], smopts(opts, 'display', 0), varargin{:});

% Jacobian matrix approximation to the fine model D
% initialize fine model Jacobian approximation, D = c'(z)
[c D] = feval(coarse, z, varargin{:});
Hc = smmerit(H, c);

% define current iterate before entering the main loop
xk = x;
xn = 1 + norm(x);
fk = f;
Hfk = Hf;
zk = z;
ck = c;
Hck = Hc;

% display some progress information
if opts.display, disp(sprintf('k: %d  Hc: %g', k, Hc)); end

if nargout > 3
  global trace
  trace = struct('k', k, 'x', x, 'z', z, 'f', f, 'h', norm(x-x0, inf), 'dx', dx, 'rho', [], 'obj', Hc, 'c', c, 'hf', Hf); % trace variable
end

if k >= opts.kmax
  stop = 2;
end

%%% main loop %%%

while ~stop

  % optimize mapped_model, xnew = arg min_x max{c(p_k(x))}
  % subject to linear constraints (user and trust region constraints),
  
  % define trust region constraints
  % xk-dx <= x <= xk+dx
  Atr = [-eye(n); eye(n)];
  btr = [-(xk-dx); xk+dx];
  
  if opts.display >= 2
    disp('Solving min_x H(c(p_k(x)))')
  end
  
  % call direct optimization
  [x, s, Hs] = smdirect(H, @mapped_model, xk, [A;Atr], [b;btr], eq, smopts(opts, 'display', 0,'dx', max(dx/3, 2*opts.epsilon)), ...  % arguments for smdirect
    coarse, xk, zk, B, varargin{:}); % this line is arguments for mapped_model
  
  % the tentative step
  h = x - xk;
      
  % check if step length is too small
  if norm(h) <= opts.epsilon * xn
    stop = 1;
    break  % break the loop now
  end
          
  % DHs = -Delta S_k(x) (= H(c(z_k)) - H(s_k(x_k + h)))
  DHs = Hck - Hs;
  
  if DHs <= 0

    % step is not accepted due to insufficient decrease in mapped model
    stop = 4;  % no decrease in local model
    break
    
  else

    % evaluate fine model at x
    if opts.display > 1
      disp('evaluating fine model')
    end
    f = feval(fine, x, varargin{:});
    Hf = smmerit(H, f);
    k = k + 1;
    
    % parameter extraction
    if opts.display > 1
      disp('parameter extraction')
    end
    z = parameter_extraction(x, coarse, xk, zk, B, zast, f, D, smopts(opts, 'display', 0), varargin{:});
    c = feval(coarse, z, varargin{:});
    Hc = smmerit(H, c);
    
    % DHf = H(c(z_k)) - H(c(z))
    DHc = Hck - Hc;
    
    % boolean determining if the tentative step is accepted
    accept = (DHc >= opts.delta1 * DHs);

    % Update trust region
    
    % calculate gain
    rho = DHc / DHs;

    % set new trust region size
    if rho <= opts.delta2
      dx = opts.k1 * dx;
      % check if trust region is smaller than minimum allowed step length
      if dx <= opts.epsilon * xn
        stop = 3;  % stop before next iteration
      end
    end
    if rho >= opts.delta3, dx = min(dxmax, opts.k2 * dx); end
          
    % Broyden update of mapping approximation
    Bnew = broyunfac(B, xk, x, zk, z);
    if opts.display > 2, disp(sprintf('||B-Bnew|| = %g', norm(B-Bnew, 'fro'))); end
    B = Bnew;
    
    % Broyden update of the linear approximation of the fine model
    Dnew = broyunfac(D, xk, x, fk, f);
    if opts.display > 2, disp(sprintf('||D-Dnew|| = %g', norm(D-Dnew, 'fro'))); end
    D = Dnew;    
  end
  
  if accept
    % accept the tentative iterate
    xk = x;
    xn = 1 + norm(x);
    fk = f;
    Hfk = Hf;
    zk = z;
    ck = c;
    Hck = Hc;
    
    if opts.display > 1, disp('step accepted'); end
    
  else
    
    if opts.display > 1, disp('step not accepted'); end
    
  end    
    
  % display some information
  if opts.display, disp(sprintf('k: %d  Hc: %g  DHc: %g  DHs: %g  dx: %g', k, Hc, DHc, DHs, dx)); end
  
  % update the trace variable
  if nargout > 3
    trace = struct('k', [trace.k k], 'x', [trace.x x], 'z', [trace.z z], 'f', [trace.f f], 'h', [trace.h norm(h, inf)], 'dx', [trace.dx dx], 'rho', [trace.rho rho], 'obj', [trace.obj Hc], 'c', [trace.c c], 'hf', [trace.hf Hf]); % trace variable
  end

  % test termination condition
  if k >= opts.kmax, stop = 2; end
  
end

if opts.display
  switch stop
  case 1
    disp('stop: step length too small')
  case 2
    disp('stop: reached the maximum allowed number of function evaluations')
  case 3
    disp('stop: trust region too small')
  case 4
    disp('stop: no decrease in local model')
  case 6
    disp('stop: solving of subproblem failed')
  end
end
