function opts = smopts(varargin)
% Returns a structure with auxiliary options for the space mapping algorithms
% 
% Call
%
%   opts = smopts(key1, value1, key2, value2, ...)
%
% optional user provided key/value pairs are inserted to the structure,
% overriding the default values.
%
% Alternative call
%
%   opts = smopts(opts, key1, value1, key2, value2, ...)
%
% the provided key/value pairs overwrite the values in the provided opts struct
%
% Note: user provided values are not checked for validity
%
%
% See the source code for a list of possible key/value pairs.

% js@imm.dtu.dk, January 26, 2003
% modified by Pernille Brock, September 30, 2004

% --------------------------------------------------------------------------------------

% A # is short notation for space mapping algorithm number #, defined as follows
%
% 1   SMO   orginal space mapping formulation
% 2   SMON  new space mapping formulation with mapped coarse model
% 3   SMH   space mapping hybrid algorithm
% 4   SMHO  space mapping hybrid algorithm with orthogonal steps
% 5   SMHC  space mapping hybrid algorithm with response correction
% 6   SMIS  space mapping algorithm with interpolating surrogates

% --------------------------------------------------------------------------------------

% A 1 - 6
% level of display  
%    0 : off
%    1 : per iteration
%    2 : per iteration, detailed
%    3 : all information available
opts.display = 1;

% A 1 - 6
% stopping accuracy on the step-length, (stop when step length <= epsilon(1 + norm(x))
% in A 6 also stopping accuracy on the relative change in the obj. fct. 
% (stop when DHf < epsilon*(1 + abs(Hfk)))
opts.epsilon = 1e2 * eps;

% A 6
% stopping accuracy on the gradient of. obj. fct. in marquardt for optimization in 2-norm
% (stop when ||F'||_inf <= epsilon_g) 
opts.epsilon_g = sqrt(eps);

% A 1 - 6
% max. allowed number of fine model evaluations
opts.kmax = 200;

% A 1 - 5
% parameter extraction type
% 1 : normal, min_z ||c(z) - f(x)||
% 2 : regularized wrt. z*,
%       min_z { (1-lambda) * ||c(z) - f(x)|| + lambda * ||z - z*|| }
% 3 : regularized wrt. x,
%       min_z { (1-lambda) * ||c(z) - f(x)|| + lambda * ||z - x|| }
% 4 : regularized wrt. gradients approximation,
%       min_z { (1-lambda) * ||c(z) - f(x)|| + lambda * ||c'(z) - D|| },
%     where D approx. f'(x)
opts.petype = 1;  

% A 1 - 5
% mapping regularization parameter, only used if petype > 1
opts.lambda = 1e-4;

% A 1 - 5
% starting point for parameter extraction
% 1 : z*  (coarse model minimizer)
% 2 : x  (tentative iterate)
% 3 : Bk * (x - xk) + zk   (predicted by current mapping approx.)
% 4 : zk  (image of current iterate p(xk))
opts.pestart = 4;

% A 1 - 6
% initial size of the trust region,
% if parameters are not properly scaled, override this default value,
% e.g. by .1 * (1 + norm(x, 2)), x being a typical value of x
opts.dx = .1;

% A 1 - 5
% sufficient decrease ratio for step acceptance
opts.delta1 = 1e-4;

% A 1 - 5
% ratios and factors for increase and decrease of trust region
opts.delta2 = .01;    % decrease ratio
opts.k1 = .5;         % decrease factor
opts.delta3 = .75;    % increase ratio
opts.k2 = 2;          % increase factor

% A 3 - 5
% perform fine model Jacobian finite difference approximation
% in stage 2 of the hybrid algorithms
% 0 : no
% 1 : yes
opts.dofinitediff = 0;

% A 3 - 5
% if yes above, perform the finite diff. approx. 
% 0  : only once
% >0 : at every 'maxuphill' uphill steps (only in stage 2)
opts.maxuphill = 0;

% A 3 - 5
% method of initialization of fine model Jacobian approximation
% 1 : Broyden update from first iteration, with sm init.
% 2 : init with surrogate model Jacobian when switching
% 3 : finite difference approximation when switching
opts.initd = 1;

% A 3, 5
% least value of w before setting to zero
opts.w_min = 1e-2;

% A 3, 5
% reduce factor of w  (K3 in the algorithm)
opts.w_reduce = .5;

% A 3 - 5
% max allowed number of iterations where w is not reduced
% (note that A 4 does not start counting till after n iterations)
opts.max_w_not_reduced = 0;  % if less than the problem dimension n, n is used instead

% A 4
% method for generating orthogonal steps
% 1 : FP's method described in the thesis
% 2 : "equality constraint" method
opts.ortho_meth = 1;

% A 4
% method for scaling orthogonal steps
% 1 : scaling depends on ||x||
% 2 : scaling depends on dx
opts.ortho_scale_type = 2;

% A 4
% scaling constant for orthogonal step
opts.ortho_scale = 0.5;

% A 6
% relative step length used in fdgrad for finite difference appr. wrt. the
% mapping parameters
opts.eta = 1e-5;

% A 6
% determines the number of mapping parameters
% 0 : a full input mapping parameter matrix A is used (n*n + n + 1 mapping parameters)
% 1 : a diagonal input mapping parameter matrix A is used (2n + 1 mapping parameters)
opts.diag_a = 0;

% A 6
% determine weighting factor ratio for the residual in the mapping parameter extraction 
% opts.weight_f: weighting factor ratio on the normalized function value residual
%                opts.weight_f = w_1/w_2, where 
%                   w_1 is the weight on the function value residual
%                   w_2 is the weight on the gradient residual
%                recommended value: opts.weight_f <= 1
opts.weight_f = 1;

% --------------------------------------------------------------------------------------

% insert the user provided key/value pairs
if nargin > 0
  j = 0;
  if isstruct(varargin{1})
    % user has provided an opts struct
    j = 1;
    opts = varargin{1}; % override default values
  end
  if mod(nargin-j,2)~=0
    error('not an even number of input parameters, input should be key/value pairs')
  end
  for i = j+1 : 2 : nargin
    if ~isfield(opts, lower(varargin{i})), error(sprintf('argument %d is not a field in the auxiliary options structure', i)); end
    opts = setfield(opts, varargin{i}, varargin{i+1});  % override default value
  end
end
