function smplot(p, trace, opts)
% Draw some plots based on the trace variable 
% and display table with level of accuracy
%
%   smplot(p, trace, opts)
%
% Input
%
% p     : a problem structure as defined by smprob
% trace : trace from iteration process
% opts  : options used to obtain theresults in trace
%         if not present the default opts are obtained from smopts
%
% Output plots (depending on algorithm number in smrun)
%
% figure(1):
%   Algorithm 1-7
%   - Performance measured in merit function H(f(x))-H(f(x*))
%   - Performance measured in iterate ||x-x*||_2
% figure(2)
%   Algorithm 1-7
%   - Trust region radius dx and inf-norm of the step length
% figure(3)
%   Algorithm 1-7
%   - Fine model response function at x* and at the space mapping solution
% figure(4)
%   Algorithm 1-5
%   - The 2-norm of the difference between the space mapping
%     and the coarse model minimizer
%   Algorithm 3-5
%   - Transition parameter w
%   Algorithm 6 (smis)
%   - The approximation errors from linear Taylor model
%     and the optimal surrogate model
% figure(5)
%   Algorithm 5 (smhc)
%   - Response correction factors
%
% Table displays the level of accuracy for each iteration 
%   level of accuracy = H(f(x_k)) - H(f(x*)) 
%
% js@imm.dtu.dk, January 22, 2003.
% modidfied by Pernille Brock, IMM, DTU, September 21, 2004 

close all

if nargin < 3
    opts = smopts;
end

figure(1), clf
% plots for algorithm 1-7

if isfield(trace, 'hf')
    Hf = trace.hf;
else
    Hf = trace.obj;
end
[Hfmin, index] = min(Hf);

subplot(2,1,1)
if ~isempty(p.xast) | ~isempty(p.fast)
    if isempty(p.fast), p.fast = feval(p.fine, p.xast, p.fopts); end
    Hfast = smmerit(p.H, p.fast);
    plotdownhill( trace.k, Hf, 1, Hfast );
    title('Performance measured in merit function H(f(x))-H(f(x*))'), grid on;
else
    plotdownhill( trace.k, Hf, 1, Hfmin );
    title('Performance measured in merit function H(f(x))-min\{H(f(x))\}'), grid on;
end

subplot(2,1,2)
if ~isempty(p.xast)
    plotdownhill( trace.k, colnorm(trace.x - repmat(p.xast, 1, length(trace.k))), 1 )
    title('Performance measured in iterate ||x-x*||_2'), grid on;
else
    plotdownhill( trace.k, colnorm(trace.x - repmat(trace.x(:,index), 1, length(trace.k))), 1 )
    title('Performance measured in iterate ||x-x_{best}||_2'), grid on;
end

figure(2), clf
% plot for algorithm 1-7
semilogy(trace.k(2:end), trace.dx(1:end-1), '.', trace.k(2:end), trace.h(2:end), 'x')
title('Trust region radius and norm of the step length'), legend('dx','||h||_{\infty}',3), grid on;

figure(3), clf
% plot for algorithm 1-7
plot(1:p.m, trace.f(:,index), 'r*-', 1:p.m, p.fast, 'bo-'), grid on,
xlabel('response function'), ylabel('response')
title('Fine model response')
legend('space mapping solution','optimal solution',3)

% plots for algorithm 1-6
if isfield(trace, 'z')
    figure(4), clf          % plot for algorithm 1-5
    if (isfield(trace, 'w') | isfield(trace, 'c')), subplot(2,1,1); end
    plotdownhill(trace.k, colnorm(trace.z - repmat(p.zast, 1, length(trace.k))), 1)
    title('Convergence of space mapping ||p(x)-z*||_2'), grid on;
    if isfield(trace, 'w')
        subplot(2,1,2)      % plot for algorithm 3-5
        % hybrid algorithm
        semilogy(trace.k, trace.w + eps, '.-')
        title('Transition parameter w'), grid on;
    elseif isfield(trace, 'c')
        Hcast = smmerit(p.H, feval(p.coarse, p.zast, p.fopts))
        subplot(2,1,2)      % plot for algorithm 2
        % original space mapping
        plotdownhill(trace.k, trace.obj, 1, Hcast)
        title('Convergence of objective function H(c(p(x)))-H(c(z*))'), grid on;
    end
elseif isfield(trace, 'am') & p.n == 2
    figure(4), clf          % plot for algorithm 6
    % Load data
    [Hfbest, index] = min(trace.obj);
    xbest = trace.x(:,index);
    fbest = trace.f(:,index);
    dfbest = trace.df(:,:,index);
    if size(dfbest,2) ~= 2
        if size(dfbest,1) == 2, dfbest = dfbest';
        else, error('wrong dimension of fine model Jacobian'), end
    end
    A = trace.am(:,:,:,index);
    b = trace.bm(:,:,index);
    alpha = trace.alpham(:,:,index);
    
    % Initialize
    if isfield(p, 'b')
        xmin = -p.b(1:p.n); xmax = p.b(p.n+1:2*p.n);
    else 
        xmin = 0.5*xbest; xmax = 1.5*xbest;
    end
    np = 51; xx1 = linspace(min(xmin(1), p.x0(1)), max(xmax(1), p.x0(1)), np);
    xx2 = linspace(min(xmin(2), p.x0(2)), max(xmax(2), p.x0(2)), np);
    [x1, x2] = meshgrid(xx1, xx2); xx1 = x1(:); xx2 = x2(:);
    nfh = zeros(size(xx1)); nsh = zeros(size(xx1));
    Ef = zeros(size(xx1)); Es = zeros(size(xx1));
    
    % Calculate approximation errors from fine model and surrogate model 
    for j = 1:length(xx1)
        xh = [xx1(j) ; xx2(j) ]; h = xh - xbest;
        fh = feval(p.fine, xh, p.fopts);
        Ef(j) = norm(fh - (fbest+dfbest*h));
        sh = surrogate_model(xh, p.coarse, xbest, fbest, A, b, alpha, [1:p.mmap], opts, p.fopts );
        Es(j) = norm(fh - sh);
    end
    Ef = vec2mat(Ef,np)'; Es = vec2mat(Es,np)';
       
    % Plot approximation errors
    surf(x1,x2,Es,'FaceColor',[1 1 1]), hold on, surf(x1,x2,Ef), colormap('gray')
    title(sprintf('Approximation error for fine model Taylor appr. and surrogate model \n with expansion point x = [ %1.2f , %1.2f ]', xbest(1),xbest(2) ), 'VerticalAlignment', 'top' )
    xlabel('x_1'), ylabel('x_2'), zlabel('Norm of approximation error'), view(52.5,30)
end

if isfield(trace, 'g')
    figure(5), clf              % plot for algorithm 5
    plot(trace.g','b.-')
    ylim([-5 5])
    title('Response correction factors'), grid on;
elseif isfield(trace, 'fr')
    Fr = sqrt(2*trace.fr); 
    sFr = size(Fr);
    nsubfig = 4;         % number of subplots per figure, must be even
    nfig = ceil(sFr(1)/nsubfig);     % number of figures
    for j = 1:nfig
        figure(4+j), clf
        for i = 1:nsubfig
            nres = nsubfig*(j-1)+i;
            if nres > sFr(1), break; end
            subplot(nsubfig/2,2,i), semilogy(Fr(nres,:), 'b.-'), grid on,
            if (mod(nres,nsubfig) == 1) & (p.m == p.mmap)
                fun1 = nsubfig*(j-1)+1; fun2 = min(j*nsubfig,sFr(1));
                title(sprintf('Norm of the residual for resp. fcts. %d - %d', fun1, fun2 ))
            elseif (mod(nres,nsubfig) == 1) & (p.m ~= p.mmap)
                fun1 = nsubfig*(j-1)+1; fun2 = min(j*nsubfig,sFr(1));
                title(sprintf('Norm of the residual for resp. fcts.\n %d - %d and %d - %d', fun1, fun2, fun1+p.mmap, fun2+p.mmap ))
            end
            ylim([10^(floor(log10(min(min(Fr)))))  10^(ceil(log10(max(max(Fr)))))])
        end
    end
end

% display table with level of accuracy
acc = Hf - Hfast;
disp(sprintf('k \t Level of accuracy \n--------------------------')) 
for j = 1:length(trace.k)
    disp(sprintf('%d \t %1.4e', trace.k(j), acc(j)))
end

%---------------------------------------------------------------------------------------------------     
function plotdownhill(X, F, opt, minF)
% Plot downhill values of steps
%
%   plotdownhill(X, F, opt, minF)
%
% X       : if opt = 2 : parameters
%              opt = 1 : first axis values
% F       : objective funtion value
% minF    : wanted min of F, if not provided minF=0 is used
%
% opt = 1 : plot objective function
% opt = 2 : plot trajectory (for two dimensional problems only)

% js@imm.dtu.dk, January 22, 2003.

m = F(1);
idx = 1;
for i = 2 : length(F)
  if F(i) <= m
    idx = [idx i];
    m = F(i);
  end
end

switch opt
  case 1
    % Plot objective function
    if nargin < 4, minF = 0; end
    if isempty(X), X = 1 : length(F); end
    semilogy(X(idx), abs(F(idx) - minF) + eps, 'b-', X, abs(F - minF) + eps, 'k.')
  case 2
    % plot trajectory
    if size(X, 1) ~= 2, error('Problem is of wrong size'); end
    plot(X(1,idx),X(2,idx), 'k.-')
    hold on
    idx2 = setdiff(1:length(F), idx);
    for i = 1 : length(idx2)
      j = max(find(idx <= idx2(i)));
      plot([X(1,idx(j)), X(1,idx2(i))], [X(2,idx(j)), X(2,idx2(i))], 'b:', X(1,idx2(i)), X(2,idx2(i)), 'b.')
    end
    hold off
  otherwise
    error('Requested plottype unknown.')
end

%---------------------------------------------------------------------------------------------------  
function nx = colnorm(X,p)
% column-wise p-norm
%
%   nx = colnorm(X,p)
%
%  possible choices of p: 1, 2, inf
%
% default: p = 2


if nargin<2, p=2; end
if nargin<1, error('Insufficient number of input arguments.'); end
if size(X,1)<2, nx = abs(X); return; end

switch p
case 1
   nx = sum(abs(X));
case 2
   nx = sqrt(sum(X.^2));
case inf
   nx = max(abs(X));
otherwise
   error(sprintf('Norm %d not defined',p))
end