function [prob, opts] = smprob(num, opts)
% Retrieve problem definition
%
%   [prob, opts] = smprob(num, opts)
%
% num : number of a predefined test problem,
%
%   Tranmission Line Transformer problems
%   1  - Two-Section Impedance Transformer
%   2  - Seven-Section Impedance Transformer
%
%   Piston problem
%   3  - Piston simulator
%
%   Rosenbrock problem
%   4  - Rosenbrock function, with linear transformation ( p(x*) = z* )
%   
%   Parallel Resonator problems
%   5  - exact linear mapping ( p(x*) = z* )
%   6  - exact non-linear mapping ( p(x*) = z* )
%   7  - inexact non-linear mapping (different topology)
%   8  - inexact non-linear mapping
%
%   Quadratic functions
%   9  - quad (coarse responses shifted up)
%   10 - quad (coarse responses shifted down)
%
%
%
%  opts : options for space mapping routines, see smopts

% js@imm.dtu.dk, January 26, 2003

% get smprobs directory
SMPROBPATH = pwd;

% add new paths
addpath( [ SMPROBPATH ]                  , ...
         fullfile(SMPROBPATH,'tlt')      , ...
         fullfile(SMPROBPATH,'piston')   , ...
         fullfile(SMPROBPATH,'rosen')    , ...
         fullfile(SMPROBPATH,'rlc')      , ...
         fullfile(SMPROBPATH,'quad')  )

clear SMPROBPATH

if nargin < 1
  error('no problem number supplied by input argument')  
end

% if opts not provided, use smopts if it exists,
% otherwise use an empty matrix for default options
if nargin < 2
  if exist('smopts')
    opts = smopts;
  else
    opts = [];
  end
end

% The prob structure consists of the fields
%
% x0     : starting point for optimization
% xast:  : fine model minimizer
% zast   : coarse model minimizer
% fine   : fine model function handle
% coarse : coarse model function handle
% n      : number of variables (length(x0))
% fopts  : options passed to models
% m      : number of response functions
% mmap   : number of response functions for mapping parameter extraction
%          mmap = 0.5*m in the case of inf-norm minimization, where f = [f ; -f]
% fast   : optimal fine model response (feval(fine,xast,fopts))
% A      : matrix of linear constraints
% b      : vector of linear constraints
% eq     : number of equality constraints
% H      : merit function, (2, inf, 'minimax')
        
switch num
  case 1
    %
    % Two-Section Impedance Transformer
    %
                 
    prob.xast = [74.233240841879066 79.265788011076197]'; % fine model optimizer
    prob.zast = [90 90]';                                 % coarse model optimizer
    prob.x0 = prob.zast;                                  % initial parameters 
    
    prob.fine = @tlt2f;
    prob.coarse = @tlt2c;
    
    % bounds, lower <= x <= upper
    prob.n = length(prob.x0);
    lower = repmat(0.0, prob.n, 1);
    upper = repmat(200.0, prob.n, 1);
    
    % frequency points of interest
    prob.fopts.state = [.5 : .1 : 1.5]';  % GHz
    prob.m = length(prob.fopts.state);
    prob.mmap = prob.m;
    prob.fast = feval(prob.fine, prob.xast, prob.fopts);    
   
    opts.dx = 20;  % suggested initial size of the trust region
    
  case 2
    %
    % Seven-Section Impedance Transformer
    % 
    
    % optimal parameters (fine model)
    prob.xast = [81.65148220976137
                   85.52171481989218
                   87.54698544710705
                   88.62554722596209
                   89.25567599948434
                   89.95304587498956
                   84.87655093368730];
                 
    % optimal parameters (coarse model)
    prob.zast = repmat(90, 7, 1);

    prob.x0 = prob.zast; % initial parameters (coarse model optimizer)
    
    % function handles
    prob.fine = @tlt7f;
    prob.coarse = @tlt7c;
    
    % bounds, lower <= x <= upper
    prob.n = length(prob.x0);
    lower = repmat(0.0, prob.n, 1);
    upper = repmat(200.0, prob.n, 1);
    
    % frequency points of interest
    prob.fopts.state = [1 : .1 : 7.7]';  % GHz
    prob.m = length(prob.fopts.state);
    prob.mmap = prob.m;
    prob.fast = feval(prob.fine, prob.xast, prob.fopts);

    opts.dx = 9;  % suggested initial size of the trust region
    
  case 3
    %
    % Piston simulator
    %
    
    prob.x0 = [151.893242617925 134.857398834676]';    % initial parameters 
    prob.xast = [100 175]';                            % fine model optimizer
    prob.zast = [151.893242617926 134.857398834676]';  % coarse model optimizer
    
    prob.fine = @piston2f;
    prob.coarse = @piston2c;
    
    % bounds, lower <= x <= upper
    prob.n = length(prob.x0);
    lower = repmat(50, prob.n, 1);
    upper = repmat(250, prob.n, 1);
    prob.fopts.state = (125 : 125 : 2500)';    % evaluation times
    prob.fopts.target = piston([200 100 175 175 100 200]', prob.fopts.state); % target response (specifications)
    prob.m = 2 * length(prob.fopts.state);     % number of response functions in minimax objective
    prob.mmap = 0.5*prob.m;                    % number of non-equivalent response functions
    prob.fast = zeros(prob.m,1);

    opts.dx = 40;  % suggested initial size of the trust region
    
  case 4
    %
    % Rosenbrock function
    %
    
    prob.xast = [1 1]'; % optimal parameters (fine model)
    
    prob.fine = @rosenf;
    prob.coarse = @rosenc;
    
    % bounds, lower <= x <= upper
    prob.n = length(prob.xast);
    lower = repmat(-5, prob.n, 1);
    upper = repmat(5, prob.n, 1);
    prob.m = 4;                         % number of response functions in minimax objective
    prob.mmap = 2;                      % number of non-equivalent response functions
    
    % linear parameter transformation between models
    prob.fopts.A = [1 2; 5 0];
    prob.fopts.b = [-3 1]';
    prob.zast = prob.fopts.A \ (prob.xast - prob.fopts.b);
    prob.x0 = prob.zast;
    prob.fast = feval(prob.fine, prob.xast, prob.fopts);

    opts.dx = .1;  % suggested initial size of the trust region
    
  case {5, 6, 7, 8}
    %
    % RLC parallel resonator
    %
        
    switch num
    case 5
      prob.xast = [45.00001698031934 0.15897712553089 11.81367831283344]'; % optimal parameters (fine model 1)
      prob.fopts.fine = @rlcf1;
    case 6
      prob.xast = [55.5552683308902 0.158976728674343 15.8137227813463]'; % optimal parameters (fine model 2)
      prob.fopts.fine = @rlcf2;
    case 7
      prob.xast = [49.84659157060781 0.12703895354361 20.30101935313988]'; % optimal parameters (fine model 3)
      prob.fopts.fine = @rlcf3;
    case 8
      prob.xast = [52.91750241552343 0.15970347668329 17.61954497546049]'; % optimal parameters (fine model 3)
      prob.fopts.fine = @rlcf4;
    end
    
    prob.fopts.coarse = @rlcc;
    prob.zast = [50.0000145182842 0.158977121616819 17.8136787514125]'; % coarse model minimizer
    prob.x0 = prob.zast; % initial parameters (coarse model minimizer)
    
    prob.fine = @rlcfspec;
    prob.coarse = @rlccspec;
    
    % bounds, lower <= x <= upper
    prob.n = length(prob.x0);
    lower = repmat(0.1, prob.n, 1);
    upper = repmat(200.0, prob.n, 1);
    
    % frequency points of interest
    prob.fopts.state = linspace(1, 5, 15)'; % frequency in GHz
    prob.m = length(prob.fopts.state);      % number of response functions (frequency points)
    prob.mmap = prob.m;                     % number of non-equivalent response functions
    prob.fast = feval(prob.fine, prob.xast, prob.fopts);
    
    opts.dx = .1;  % suggested initial size of the trust region
    
  case {9, 10}
    %
    % Three quadratic functions
    %
  
    prob.xast = [0 0]';
    prob.zast = [0.1 0.1]';
    prob.x0 = prob.zast;
    
    prob.fine = @quadraticf;
    prob.coarse = @quadraticc;
    
    % bounds, lower <= x <= upper
    prob.n = length(prob.x0);
    lower = repmat(-10, prob.n, 1);
    upper = repmat(10, prob.n, 1);

    prob.m = 3;                             % number of response functions
    prob.mmap = prob.m;                     % number of non-equivalent response functions
    prob.fast = repmat(-2, prob.m, 1);
    % shift coarse model response up (num = 9) or down (num = 10)
    if num == 9
      prob.fopts = 0.1;
    else 
      prob.fopts = -0.1;
    end     
    
    opts.dx = 5;    
     
  otherwise
    error(sprintf('Problem number %d not defined.', num))
end

% define linear constraints from the bounds
% A * x <= b, first eq rows are equality constraints
prob.A = [-eye(prob.n); eye(prob.n)];
prob.b = [-lower; upper];
prob.eq = 0;    % number of equality constraints

prob.H = 'minimax';

if exist('smauxopts')
  % overwrite defualt option values
  opts = smauxopts(opts, 'kmax', 100 * prob.n, 'dx', .1 * (1 + norm(prob.x0, 2)), 'maxuphill', prob.n);
end
