function [p, trace, opts] = smrun(num, algo, opts)
% Run a space mapping optimization algorithm
%
%    [p, trace, opts] = smrun(num, algo, opts)
%
% Input
% 
% num : number of a predefined test problem,
%
%   Tranmission Line Transformer problems
%   1  - Two-Section Impedance Transformer
%   2  - Seven-Section Impedance Transformer
%
%   Piston problem
%   3  - Piston simulator
%
%   Rosenbrock problem
%   4  - Rosenbrock function, with linear transformation ( p(x*) = z* )
%   
%   Parallel Resonator problems
%   5  - exact linear mapping ( p(x*) = z* )
%   6  - exact non-linear mapping ( p(x*) = z* )
%   7  - inexact non-linear mapping (different topology)
%   8  - inexact non-linear mapping
%
%   Quadratic functions
%   9  - quad (coarse responses shifted up)
%   10 - quad (coarse responses shifted down)
%
%
% algo : algorithm number,
%
%   1   - SMO   orginal space mapping formulation
%   2   - SMON  new space mapping formulation with mapped coarse model
%   3   - SMH   space mapping hybrid algorithm
%   4   - SMHO  space mapping hybrid algorithm with orthogonal steps
%   5   - SMHC  space mapping hybrid algorithm with response correction
%   6   - SMIS  space mapping algorithm with interpolating surrogate model
%   7   - direct optimization of the fine model, 1. order method
%
%   8   - direct optimization of the fine model, 2. order method (no trace)
%   9   - direct optimization coarse model, 2. order method      (no trace)
%
% opts : options for the optimization algorithm
%        if not is supplied the defaults are obtained from smopts
%
%
% Output
%
% p     : struct defining the problem (from smprob)
% trace : a structure with a trace of the optimization process
% opts  : the options used when calling the algorithm 
%         (may have been modified by smprob from the opts given as input)

% js@imm.dtu.dk, January 27, 2003
% modified by Pernille Brock, IMM, DTU, September 9, 2004

if nargin < 2, error('At least two input arguments should be given'); end
if nargin < 3, opts = smopts; end

[p, opts] = smprob(num, opts);

switch algo
case 1
  fun = @smo;
case 2
  fun = @smon;
case 3
  fun = @smh;
case 4
  fun = @smho;
case 5
  fun = @smhc;
case 6
  fun = @smis;
end

switch algo
case {1,2,3,4,5}
  if nargout > 1
      [x, fk, Hfk, stop, trace] = feval(fun, p.H, p.fine, p.coarse, p.x0, p.A, p.b, p.eq, opts, p.fopts);
  else
      [x, fk, Hfk, stop] = feval(fun, p.H, p.fine, p.coarse, p.x0, p.A, p.b, p.eq, opts, p.fopts);
  end
case 6
  if nargout > 1
      [x, fk, Hfk, stop, trace] = feval(fun, p.H, p.mmap, p.fine, p.coarse, p.x0, p.A, p.b, p.eq, opts, p.fopts);
  else
      [x, fk, Hfk, stop] = feval(fun, p.H, p.mmap, p.fine, p.coarse, p.x0, p.A, p.b, p.eq, opts, p.fopts);
  end  
case 7
  if nargout > 1
      [x, fk, Hfk, stop, trace] = direct(p.H, p.fine, p.x0, p.A, p.b, p.eq, opts, p.fopts);
  else
      [x, fk, Hfk, stop] = direct(p.H, p.fine, p.x0, p.A, p.b, p.eq, opts, p.fopts);
  end
case 8
  x = smdirect(p.H, p.fine, p.x0, p.A, p.b, p.eq, opts, p.fopts);
  trace = x;
case 9
  x = smdirect(p.H, p.coarse, p.x0, p.A, p.b, p.eq, opts, p.fopts);
  trace = x;
end
