function [f, df] = fdgrad(func, x, f, eta, varargin)
%FDGRAD  Forward difference gradient/Jacobian approximation
%
% Call
%      [f, df] = fdgrad(func, x, f, eta, P1, P2, ...)
%
% where
%   func    : function handle to function obeying the call
%             f = feval(func, x, P1, P2, ...)
%   x       : point of approximation
%   f       : function value(s) at x, f(x)
%             if empty at input, it is calculated by the routine
%   eta     : noise scale, if omitted default is sqrt(eps) (minimum allowed value)
%   P1, ... : optional parameters passed to func
%   df      : forward difference approximation to the gradient at x
%             if f is a vector function, df is the Jacobian matrix
%
% Calculates a forward difference approximation to the gradient for
% the function func. If func is a vector function, the Jacobian
% matrix (transpose of the gradients) is returned in df.

% js@imm.dtu.dk, December 3, 2002
% Modified by Pernille Brock, IMM, DTU, September 7, 2004

% Modelled after algorithms A5.4.1 and A5.6.3 in
% "Numerical Methods for Unconstrained Optimization and Nonlinear Equations"
% by J.E. Dennis, Jr. and Robert B. Schnabel

% set f
if nargin < 3, f = []; end
if isempty(f), f = feval(func, x, varargin{:}); end
f = f(:);  % convert to column vector

% set noise scale, modified
if isempty(eta), eta = sqrt(eps);
else eta = max(sqrt(eps), eta);
end

% reserve space for df
df = zeros(length(f), length(x));

for j = 1 : length(x)
  tempj = x(j);
  
  s = sign(tempj);
  if s == 0, s = 1; end
  
  stepsizej = eta * max(abs(tempj), eta) * s;   % modified

  x(j) = tempj + stepsizej;
  stepsizej = x(j) - tempj;
  
  fj = feval(func, x, varargin{:});
  
  df(:, j) = (fj(:) - f) / stepsizej;

  x(j) = tempj;
end

if length(f) == 1, df = df'; end % return the gradient for scalar functions
