function [f, df] = tlt2f(x, t)
% Two-Section Impedance Transformer (fine model)

% js@imm.dtu.dk, January 9, 2003

if nargout > 1
  [f, df] = fdgrad(@ZTRANf, x, [], 0, t.state);
else
  f = feval(@ZTRANf, x, t.state);
end
  
function mS11 = ZTRANf(L, freq)
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
% Jose E. Rayas-Sanchez                                     November 1, 2000
% Simulation Optimization Systems Research Laboratory    McMaster University
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
%           Two-Section Impedance Transformer (fine model)
%
% This function calculates the magnitude of the reflection coefficient of a
% two-section impedance transformer with capacitively-loaded ideal 
% transmission lines.
% Usage: mS11 = ZTRAN2f(L,freq)
%        L:    Vector of lengths of the transmission lines. L = [L1 L2]', 
%              with L1 and L2 in degrees (electrical lengths).
%        freq: Vector of frequencies in GHz.
%        mS11: Vector of responses (magnitude of S11) at each frequency.
%

% optimal parameters L = [74.23324113038596  79.26578771646511]'
% for freq = [.5:.1:1.5]';

Z(1) = 4.47213599101694; % Characteristic impedance of first transmission line.
Z(2) = 2.23606799550847; % Characteristic impedance of second transmission line.
c = 3.0e8;            % Speed of light in free-space.
Fo = 1e9;             % Reference frequency for electrical length.
L = (c/(360*Fo))*L;   % Converting electrical length to physical length.
Ro = 1;               % Reference impedance.
ZL = 10;              % Load impedance (connected to first T.L.).
Fp = length(freq);    % Number of frequency points requested.    
mS11 = zeros(Fp,1);
Omega = 2.*pi.*1e9.*freq;
Lambda = c./(1e9.*freq);
Beta = (2*pi)./Lambda;

C1=10e-12;             % Capacitors.
C2=10e-12;
C3=10e-12;
Zc1=1./(j.*Omega.*C1); % Capacitor impedances.
Zc2=1./(j.*Omega.*C2);
Zc3=1./(j.*Omega.*C3);

factor1 = j*tan(Beta.*L(1));
factor2 = j*tan(Beta.*L(2));
Zp1 = (ZL.*Zc1)./(ZL+Zc1);
Zp2 = Z(1).*((Zp1+Z(1).*factor1)./(Z(1)+Zp1.*factor1));
Zp3 = (Zp2.*Zc2)./(Zp2+Zc2);
Zp4 = Z(2).*((Zp3+Z(2).*factor2)./(Z(2)+Zp3.*factor2));
Zin = (Zp4.*Zc3)./(Zp4+Zc3);
S11 = (Zin-Ro)./(Zin+Ro);
mS11 = abs(S11);
