function [f, df] = tlt7f(x, t)
% Seven-Section Impedance Transformer (fine model)

% js@imm.dtu.dk, January 13, 2003

if nargout > 1    
  [f, df] = fdgrad(@ZTRANf, x, [], 0, t.state);
else
  f = feval(@ZTRANf, x, t.state);
end

function mS11 = ZTRANf(L,freq)
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
% Ahmed S. Mohamed                                                 May, 2001
% Simulation Optimization Systems Research Laboratory    McMaster University
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
%          Seven-Section Impedance Transformer (fine model)
%
% This function calculates the magnitude of the reflection coefficient of a
% two-section impedance transformer with ideal transmission lines.
% Usage: mS11 = ZTRAN7f(L,freq)
%        L:    Vector of lengths of the transmission lines. L = [Li]', 
%              i = 1,...,7, in degrees (electrical lengths).
%        freq: Vector of frequencies in GHz.
%        mS11: Vector of responses (magnitude of S11) at each frequency. It
%              is a row vector.
%

% optimal parameters L = [81.6514822097614 85.5217148198922 87.546985447107 88.6255472259621 89.2556759994843 89.9530458749896 84.8765509336873]'
% for freq = [1:.1:7.7]';

Z = [91.94454481062100  85.52387852113699  78.15262602058000  70.71067811865500 63.97737676381300  58.46320450450900  54.38060529092400]; % Vector of charactteristic impedance

Cap = [2.5e-14 2.5e-14 2.5e-14 2.5e-14 2.5e-14 2.5e-14 2.5e-14 2.5e-14]'; % vector of parasitic capacitances
C = 3.0e8;
Fo = 4.35e9;          % Reference frequency for electrical length.
L = (C/(360*Fo))*L;   % Converting electrical length to physical length.
freq = 1e9 * freq;
RL = 100;             % load impedance
Ro = 50;              % source impedance
N = length(L);

Omega = (2 * pi) * freq;      % angular frequencies
Lambda = C ./ freq;           % wave length
Beta = (2 * pi) ./ Lambda;    % propagation constant
factor = j * tan(Beta * L');  % factor matrix
Zc = 1 ./ (j * Omega * Cap'); % capacitance impedance matrix
Zin = RL;
for i = 1 : N + 1
  Zin = (Zin .* Zc(:,i)) ./ (Zin + Zc(:,i)); % capacitor in parallel with the impedance
  if(i <= N) Zin = Z(i) * ((Zin + Z(i) * factor(:,i)) ./ (Z(i) + Zin .* factor(:,i))); end % Get new input impedance
end
S11 = (Zin - Ro) ./ (Zin + Ro); % S11
mS11 = abs(S11);                % absolute value of the scattering parameters
